@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -35,9 +35,9 @@
 #include "oval_parser_impl.h"
 #include "oval_string_map_impl.h"
 
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -212,7 +212,7 @@ static int _oval_affected_parse_tag(xmlTextReaderPtr reader, struct oval_parser_
 {
 	struct oval_affected *affected = (struct oval_affected *)user;
 	int return_code;
-	xmlChar *tagname = xmlTextReaderName(reader);
+	xmlChar *tagname = xmlTextReaderLocalName(reader);
 	//xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 
 	if (strcmp((char *)tagname, "platform") == 0) {
