@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -27,6 +27,9 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#include <string.h>
+#include <time.h>
+#include "config.h"
 #include "oval_definitions_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
@@ -34,11 +37,9 @@
 #include "oval_system_characteristics_impl.h"
 #include "oval_probe_impl.h"
 #include "oval_results_impl.h"
-#include <string.h>
-#include <time.h>
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -51,6 +52,7 @@ typedef struct oval_definition_model {
 	struct oval_string_map *state_map;
 	struct oval_string_map *variable_map;
 	bool is_locked;
+        char *schema;
 } oval_definition_model_t;
 
 typedef struct oval_syschar_model {
@@ -60,12 +62,14 @@ typedef struct oval_syschar_model {
 	struct oval_string_map *sysdata_map;
 	struct oval_string_map *variable_binding_map;
 	bool is_locked;
+        char *schema;
 } oval_syschar_model_t;
 
 struct oval_results_model {
 	struct oval_definition_model *definition_model;
 	struct oval_collection *systems;
 	bool is_locked;
+        char *schema;
 };
 
 typedef struct oval_results_model oval_results_model_t;
@@ -86,10 +90,50 @@ static void _oval_agent_scan_entity_for_references(struct oval_entity *entity,
 static void _oval_agent_scan_set_for_references(struct oval_setobject *set,
 						struct oval_string_map *objmap,
 						struct oval_string_map *sttmap, struct oval_string_map *varmap);
+
+/*static char * _oval_generate_schema_location(const char * version);*/
 /* End of static functions definitions
  * */
 /***************************************************************************/
 
+
+/* TODO: Use this to generate links
+static char * _oval_generate_schema_location(const char * version)
+{
+
+        char * schema = oscap_alloc(sizeof(char)*(strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_SX)+2));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_DEF_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_PX));
+        strncat(schema, version, strlen(version));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_DEF_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_SX));
+        strncat(schema, " ", 1);
+
+        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_IND_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_IND_SX)+1));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_IND_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_IND_PX));
+        strncat(schema, version, strlen(version));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_IND_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_IND_SX));
+        strncat(schema, " ", 1);
+
+        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_SX)+1));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_UNX_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_PX));
+        strncat(schema, version, strlen(version));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_UNX_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_SX));
+        strncat(schema, " ", 1);
+
+        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_SX)+1));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_LNX_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_PX));
+        strncat(schema, version, strlen(version));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_LNX_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_SX));
+        strncat(schema, " ", 1);
+
+        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_SX)+1));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_CMN_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_PX));
+        strncat(schema, version, strlen(version));
+        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_CMN_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_SX));
+
+
+        return schema;
+}*/
+
 /* failed   - NULL 
  * success  - oval_definition_model 
  * */
@@ -105,6 +149,7 @@ struct oval_definition_model *oval_definition_model_new()
 	newmodel->test_map = oval_string_map_new();
 	newmodel->variable_map = oval_string_map_new();
 	newmodel->is_locked = false;
+        newmodel->schema = strdup(OVAL_DEF_SCHEMA_LOCATION);
 	return newmodel;
 }
 
@@ -125,7 +170,76 @@ bool oval_definition_model_is_locked(struct oval_definition_model *definition_mo
 
 bool oval_definition_model_is_valid(struct oval_definition_model * definition_model)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_definition_iterator *definitions_itr;
+	struct oval_test_iterator *tests_itr;
+	struct oval_object_iterator *objects_itr;
+	struct oval_state_iterator *states_itr;
+
+	if (definition_model == NULL)
+		return false;
+
+	/* validate definitions */
+	definitions_itr = oval_definition_model_get_definitions(definition_model);
+	while (oval_definition_iterator_has_more(definitions_itr)) {
+		struct oval_definition *definition;
+
+		definition = oval_definition_iterator_next(definitions_itr);
+		if (oval_definition_is_valid(definition) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_definition_iterator_free(definitions_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate tests */
+	tests_itr = oval_definition_model_get_tests(definition_model);
+	while (oval_test_iterator_has_more(tests_itr)) {
+		struct oval_test *test;
+
+		test = oval_test_iterator_next(tests_itr);
+		if (oval_test_is_valid(test) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_test_iterator_free(tests_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate objects */
+	objects_itr = oval_definition_model_get_objects(definition_model);
+	while (oval_object_iterator_has_more(objects_itr)) {
+		struct oval_object *obj;
+
+		obj = oval_object_iterator_next(objects_itr);
+		if (oval_object_is_valid(obj) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_object_iterator_free(objects_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate states */
+	states_itr = oval_definition_model_get_states(definition_model);
+	while (oval_state_iterator_has_more(states_itr)) {
+		struct oval_state *state;
+
+		state = oval_state_iterator_next(states_itr);
+		if (oval_state_is_valid(state) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_state_iterator_free(states_itr);
+	if (is_valid != true)
+		return false;
+
+	return true;
 }
 
 static void _oval_definition_model_clone(struct oval_string_map *oldmap,
@@ -159,6 +273,7 @@ struct oval_definition_model *oval_definition_model_clone(struct oval_definition
 	_oval_definition_model_clone(oldmodel->test_map, newmodel, (_oval_result_system_clone_func) oval_test_clone);
 	_oval_definition_model_clone
 	    (oldmodel->variable_map, newmodel, (_oval_result_system_clone_func) oval_variable_clone);
+        newmodel->schema = oscap_strdup(oldmodel->schema);
 	return newmodel;
 }
 
@@ -172,11 +287,15 @@ void oval_definition_model_free(struct oval_definition_model *model)
 	oval_string_map_free(model->test_map, (oscap_destruct_func) oval_test_free);
 	oval_string_map_free(model->variable_map, (oscap_destruct_func) oval_variable_free);
 
+        if (model->schema != NULL)
+            oscap_free(model->schema);
+
 	model->definition_map = NULL;
 	model->object_map = NULL;
 	model->state_map = NULL;
 	model->test_map = NULL;
 	model->variable_map = NULL;
+        model->schema = NULL;
 
 	oscap_free(model);
 }
@@ -196,6 +315,7 @@ struct oval_syschar_model *oval_syschar_model_new(struct oval_definition_model *
 	newmodel->sysdata_map = oval_string_map_new();
 	newmodel->variable_binding_map = oval_string_map_new();
 	newmodel->is_locked = false;
+        newmodel->schema = oscap_strdup(OVAL_SYS_SCHEMA_LOCATION);
 
 	/* check possible allocation problems */
 	if ((newmodel->syschar_map == NULL) ||
@@ -224,7 +344,43 @@ bool oval_syschar_model_is_locked(struct oval_syschar_model *syschar_model)
 
 bool oval_syschar_model_is_valid(struct oval_syschar_model * syschar_model)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_syschar_iterator *syschars_itr;
+	struct oval_sysinfo *sysinfo;
+	struct oval_definition_model *definition_model;
+
+	if (syschar_model == NULL)
+		return false;
+
+	/* validate sysinfo */
+	sysinfo = oval_syschar_model_get_sysinfo(syschar_model);
+	if (oval_sysinfo_is_valid(sysinfo) != true)
+		return false;
+
+	/* validate definition_model */
+	definition_model = oval_syschar_model_get_definition_model(syschar_model);
+	if (oval_definition_model_is_valid(definition_model) != true)
+		return false;
+
+	/* validate syschars */
+	syschars_itr = oval_syschar_model_get_syschars(syschar_model);
+	while (oval_syschar_iterator_has_more(syschars_itr)) {
+		struct oval_syschar *syschar;
+
+		syschar = oval_syschar_iterator_next(syschars_itr);
+		if (oval_syschar_is_valid(syschar) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_syschar_iterator_free(syschars_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate variable_bindings */
+	// todo
+
+	return true;
 }
 
 static void _oval_syschar_model_clone_variable_binding(struct oval_variable_binding *old_binding,
@@ -266,6 +422,7 @@ struct oval_syschar_model *oval_syschar_model_clone(struct oval_syschar_model *o
 	struct oval_sysinfo *old_sysinfo = oval_syschar_model_get_sysinfo(old_model);
 	struct oval_sysinfo *new_sysinfo = oval_sysinfo_clone(new_model, old_sysinfo);
 	oval_syschar_model_set_sysinfo(new_model, new_sysinfo);
+        new_model->schema = oscap_strdup(old_model->schema);
 
 	return new_model;
 }
@@ -282,12 +439,15 @@ void oval_syschar_model_free(struct oval_syschar_model *model)
 		oval_string_map_free(model->sysdata_map, (oscap_destruct_func) oval_sysdata_free);
 	if (model->variable_binding_map)
 		oval_string_map_free(model->variable_binding_map, (oscap_destruct_func) oval_variable_binding_free);
+        if (model->schema)
+                oscap_free(model->schema);
 
 	model->sysinfo = NULL;
 	model->definition_model = NULL;
 	model->syschar_map = NULL;
 	model->sysdata_map = NULL;
 	model->variable_binding_map = NULL;
+        model->schema = NULL;
 	oscap_free(model);
 }
 
@@ -298,6 +458,13 @@ struct oval_definition_model *oval_syschar_model_get_definition_model(struct ova
 	return model->definition_model;
 }
 
+const char * oval_definition_model_get_schema(struct oval_definition_model * model)
+{
+        __attribute__nonnull__(model);
+
+        return model->schema;
+}    
+
 struct oval_syschar_iterator *oval_syschar_model_get_syschars(struct oval_syschar_model *model)
 {
 	__attribute__nonnull__(model);
@@ -314,6 +481,13 @@ struct oval_sysinfo *oval_syschar_model_get_sysinfo(struct oval_syschar_model *m
 	return model->sysinfo;
 }
 
+const char * oval_syschar_model_get_schema(struct oval_syschar_model * model)
+{
+        __attribute__nonnull__(model);
+
+        return model->schema;
+}    
+
 void oval_syschar_model_set_sysinfo(struct oval_syschar_model *model, struct oval_sysinfo *sysinfo)
 {
 	if (model && !oval_syschar_model_is_locked(model)) {
@@ -322,6 +496,14 @@ void oval_syschar_model_set_sysinfo(struct oval_syschar_model *model, struct ova
 		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
 }
 
+void oval_syschar_model_set_schema(struct oval_syschar_model *model, const char * schema)
+{
+	if (model && !oval_syschar_model_is_locked(model)) {
+		model->schema = oscap_strdup(schema);
+	} else
+		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+}
+
 void oval_definition_model_add_definition(struct oval_definition_model *model, struct oval_definition *definition)
 {
 	if (model && !oval_definition_model_is_locked(model)) {
@@ -331,6 +513,14 @@ void oval_definition_model_add_definition(struct oval_definition_model *model, s
 		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
 }
 
+void oval_definition_model_set_schema(struct oval_definition_model *model, const char *version)
+{
+	if (model && !oval_definition_model_is_locked(model)) {
+                model->schema = oscap_strdup(version);
+	} else
+		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
+}
+
 void oval_definition_model_add_test(struct oval_definition_model *model, struct oval_test *test)
 {
 	if (model && !oval_definition_model_is_locked(model)) {
@@ -389,19 +579,18 @@ void oval_syschar_model_add_variable_binding(struct oval_syschar_model *model, s
 		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
 }
 
-void oval_syschar_model_probe_objects(struct oval_syschar_model *syschar_model)
+int oval_syschar_model_probe_objects(struct oval_syschar_model *syschar_model)
 {
+#ifdef ENABLE_PROBES
 	__attribute__nonnull__(syschar_model);
 
-	if (syschar_model->sysinfo == NULL)
-		oval_syschar_model_get_sysinfo(syschar_model);
 	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
 
 	if (definition_model) {
-		oval_pctx_t *pctx;
+		oval_probe_session_t *sess;
 		struct oval_object_iterator *objects = oval_definition_model_get_objects(definition_model);
 
-		pctx = oval_pctx_new(syschar_model);
+		sess = oval_probe_session_new(syschar_model);
 
 		while (oval_object_iterator_has_more(objects)) {
 			struct oval_object *object = oval_object_iterator_next(objects);
@@ -409,21 +598,57 @@ void oval_syschar_model_probe_objects(struct oval_syschar_model *syschar_model)
 			struct oval_syschar *syschar = oval_syschar_model_get_syschar(syschar_model, objid);
 
 			if (syschar == NULL) {
-				syschar = oval_probe_object_eval(pctx, object);
-
+				syschar = oval_probe_object_eval(sess, object, 0);
 				if (syschar == NULL) {
 					syschar = oval_syschar_new(syschar_model, object);
 					oval_syschar_set_flag(syschar, SYSCHAR_FLAG_NOT_COLLECTED);
+					if(  oscap_err() ) {
+						oval_syschar_model_add_syschar(syschar_model, syschar);
+						oval_probe_session_destroy(sess);
+						oval_object_iterator_free(objects);
+						return 1;
+					}
 				}
 				oval_syschar_model_add_syschar(syschar_model, syschar);
 			}
 		}
 
-		oval_pctx_free(pctx);
+		oval_probe_session_destroy(sess);
 		oval_object_iterator_free(objects);
 	}
+
+	return 0;
+#else
+	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");
+        return -1;
+#endif
 }
 
+int oval_syschar_model_probe_sysinfo(struct oval_syschar_model *syschar_model) {
+#ifdef ENABLE_PROBES
+	oval_probe_session_t *sess;
+        struct oval_sysinfo *sysinfo;
+
+	sess = oval_probe_session_new(syschar_model);
+        sysinfo = oval_probe_sysinf_eval(sess);
+
+        if (sysinfo == NULL && oscap_err() ) {
+		oval_probe_session_destroy(sess);
+		return 1;
+	}
+
+        oval_syschar_model_set_sysinfo(syschar_model, sysinfo);
+        oval_sysinfo_free(sysinfo);
+	oval_probe_session_destroy(sess);
+
+	return 0;
+#else
+	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");
+        return -1;
+#endif
+}
+
+
 void oval_syschar_model_add_sysdata(struct oval_syschar_model *model, struct oval_sysdata *sysdata)
 {
 	if (model && !oval_syschar_model_is_locked(model)) {
@@ -436,7 +661,7 @@ void oval_syschar_model_add_sysdata(struct oval_syschar_model *model, struct ova
 }
 
 int oval_definition_model_import(struct oval_definition_model *model,
-				 struct oscap_import_source *source, void *user_arg)
+				 struct oscap_import_source *source)
 {
 	__attribute__nonnull__(source);
 
@@ -445,7 +670,7 @@ int oval_definition_model_import(struct oval_definition_model *model,
 	int retcode = 0;
 	if (reader) {
 		if (xmlTextReaderRead(reader) > -1) {
-			retcode = ovaldef_parser_parse(model, reader, user_arg);
+			retcode = ovaldef_parser_parse(model, reader, NULL);
 		}
 		xmlFreeTextReader(reader);
 	} else {
@@ -456,7 +681,7 @@ int oval_definition_model_import(struct oval_definition_model *model,
 	return retcode;
 }
 
-int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_import_source *source, void *user_arg)
+int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_import_source *source)
 {
 	__attribute__nonnull__(source);
 
@@ -475,7 +700,7 @@ int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_imp
 	}
 
 	xmlTextReaderRead(reader);
-	ret = ovalsys_parser_parse(model, reader, user_arg);
+	ret = ovalsys_parser_parse(model, reader, NULL);
 
 	xmlFreeTextReader(reader);
 	xmlFreeDoc(doc);
@@ -723,6 +948,7 @@ struct oval_results_model *oval_results_model_new(struct oval_definition_model *
 	model->systems = oval_collection_new();
 	model->definition_model = definition_model;
 	model->is_locked = false;
+        model->schema = oscap_strdup(OVAL_RES_SCHEMA_LOCATION);
 	if (syschar_models) {
 		struct oval_syschar_model *syschar_model;
 		for (syschar_model = *syschar_models; syschar_model; syschar_model = *(++syschar_models)) {
@@ -750,7 +976,34 @@ bool oval_results_model_is_locked(struct oval_results_model *results_model)
 
 bool oval_results_model_is_valid(struct oval_results_model * results_model)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_result_system_iterator *systems_itr;
+	struct oval_definition_model *definition_model;
+
+	if (results_model == NULL)
+		return false;
+
+	/* validate definition_model */
+	definition_model = oval_results_model_get_definition_model(results_model);
+	if (oval_definition_model_is_valid(definition_model) != true)
+		return false;
+
+	/* validate systems */
+	systems_itr = oval_results_model_get_systems(results_model);
+	while (oval_result_system_iterator_has_more(systems_itr)) {
+		struct oval_result_system *rslt_system;
+
+		rslt_system = oval_result_system_iterator_next(systems_itr);
+		if (oval_result_system_is_valid(rslt_system) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_result_system_iterator_free(systems_itr);
+	if (is_valid != true)
+		return false;
+
+	return true;
 }
 
 struct oval_results_model *oval_results_model_clone(struct oval_results_model *old_resmodel)
@@ -766,8 +1019,9 @@ struct oval_results_model *oval_results_model_clone(struct oval_results_model *o
 
 		oval_results_model_add_system(new_resmodel, new_system);
 	}
-
 	oval_result_system_iterator_free(old_systems);
+        new_resmodel->schema = strdup(old_resmodel->schema);
+
 	return new_resmodel;
 }
 
@@ -775,9 +1029,12 @@ void oval_results_model_free(struct oval_results_model *model)
 {
 	__attribute__nonnull__(model);
 
+        if (model->schema)
+                oscap_free(model->schema);
 	oval_collection_free_items(model->systems, (oscap_destruct_func) oval_result_system_free);
 	model->definition_model = NULL;
 	model->systems = NULL;
+        model->schema = NULL;
 	oscap_free(model);
 }
 
@@ -805,7 +1062,7 @@ void oval_results_model_add_system(struct oval_results_model *model, struct oval
 }
 
 struct oval_result_directives *oval_results_model_import(struct oval_results_model *model,
-							 struct oscap_import_source *source, void *client_data)
+							 struct oscap_import_source *source)
 {
 	__attribute__nonnull__(source);
 
@@ -825,7 +1082,7 @@ struct oval_result_directives *oval_results_model_import(struct oval_results_mod
 		oscap_setxmlerr(xmlGetLastError());
 		return NULL;
 	}
-	struct oval_result_directives *directives = ovalres_parser_parse(model, reader, client_data);
+	struct oval_result_directives *directives = ovalres_parser_parse(model, reader, NULL);
 
 	xmlFreeTextReader(reader);
 	xmlFreeDoc(doc);
@@ -1019,10 +1276,14 @@ xmlNode *oval_definitions_to_dom(struct oval_definition_model *definition_model,
 		root_node = xmlNewNode(NULL, BAD_CAST "oval_definitions");
 		xmlDocSetRootElement(doc, root_node);
 	}
+	xmlNewProp(root_node, BAD_CAST "xsi:schemaLocation", BAD_CAST definition_model->schema);
+
 	xmlNs *ns_common = xmlNewNs(root_node, OVAL_COMMON_NAMESPACE, BAD_CAST "oval");
+	xmlNs *ns_xsi = xmlNewNs(root_node, OVAL_XMLNS_XSI, BAD_CAST "xsi");
 	xmlNs *ns_defntns = xmlNewNs(root_node, OVAL_DEFINITIONS_NAMESPACE, NULL);
 
 	xmlSetNs(root_node, ns_common);
+	xmlSetNs(root_node, ns_xsi);
 	xmlSetNs(root_node, ns_defntns);
 
 	xmlNode *tag_generator = xmlNewChild(root_node, ns_defntns, BAD_CAST "generator", NULL);
@@ -1168,10 +1429,14 @@ xmlNode *oval_syschar_model_to_dom(struct oval_syschar_model * syschar_model,
 		root_node = xmlNewNode(NULL, BAD_CAST "oval_system_characteristics");
 		xmlDocSetRootElement(doc, root_node);
 	}
+	xmlNewProp(root_node, BAD_CAST "xsi:schemaLocation", BAD_CAST syschar_model->schema);
+
 	xmlNs *ns_common = xmlNewNs(root_node, OVAL_COMMON_NAMESPACE, BAD_CAST "oval");
+	xmlNs *ns_xsi = xmlNewNs(root_node, OVAL_XMLNS_XSI, BAD_CAST "xsi");
 	xmlNs *ns_syschar = xmlNewNs(root_node, OVAL_SYSCHAR_NAMESPACE, NULL);
 
 	xmlSetNs(root_node, ns_common);
+	xmlSetNs(root_node, ns_xsi);
 	xmlSetNs(root_node, ns_syschar);
 
 	xmlNode *tag_generator = xmlNewChild(root_node, ns_syschar, BAD_CAST "generator", NULL);
@@ -1256,7 +1521,7 @@ int oval_syschar_model_export(struct oval_syschar_model *model, struct oscap_exp
 	return retcode;
 }
 
-void oval_results_model_eval(struct oval_results_model *res_model)
+int oval_results_model_eval(struct oval_results_model *res_model)
 {
 	struct oval_result_system_iterator *systems_itr;
 
@@ -1266,10 +1531,13 @@ void oval_results_model_eval(struct oval_results_model *res_model)
 		struct oval_result_system *sys;
 
 		sys = oval_result_system_iterator_next(systems_itr);
-		oval_result_system_eval(sys);
+		if( oval_result_system_eval(sys) != 0 ) { 	/* evaluate each result_system */
+			oval_result_system_iterator_free(systems_itr);
+			return -1;
+		}
 	}
-
 	oval_result_system_iterator_free(systems_itr);
+	return 0;
 }
 
 static void _scan_for_viewable_definitions(struct oval_results_model *results_model,
@@ -1316,10 +1584,14 @@ static xmlNode *oval_results_to_dom(struct oval_results_model *results_model,
 		root_node = xmlNewNode(NULL, BAD_CAST "oval_results");
 		xmlDocSetRootElement(doc, root_node);
 	}
+	xmlNewProp(root_node, BAD_CAST "xsi:schemaLocation", BAD_CAST results_model->schema);
+
 	xmlNs *ns_common = xmlNewNs(root_node, OVAL_COMMON_NAMESPACE, BAD_CAST "oval");
+	xmlNs *ns_xsi = xmlNewNs(root_node, OVAL_XMLNS_XSI, BAD_CAST "xsi");
 	xmlNs *ns_results = xmlNewNs(root_node, OVAL_RESULTS_NAMESPACE, NULL);
 
 	xmlSetNs(root_node, ns_common);
+	xmlSetNs(root_node, ns_xsi);
 	xmlSetNs(root_node, ns_results);
 
 	xmlNode *tag_generator = xmlNewChild(root_node, ns_results, BAD_CAST "generator", NULL);
@@ -1377,39 +1649,3 @@ int oval_results_model_export(struct oval_results_model *results_model,
 	return ((xmlCode >= 1) ? 1 : -1);
 }
 
-#ifdef __STUB_PROBE		//This Macro is defined iff probes are stubbed.
-//STUB for oval_object_probe
-static int item_id = 1;
-
-struct oval_syschar *oval_probe_object_eval(oval_pctx_t * pctx, struct oval_object *object)
-{
-	__attribute__nonnull__(pctx);
-
-	char itemid[10];
-
-	struct oval_syschar_model *model = pctx->model;
-	struct oval_syschar *syschar = oval_syschar_new(model, object);
-	oval_syschar_set_flag(syschar, SYSCHAR_STATUS_NOT_COLLECTED);
-	snprintf(itemid, sizeof(item_id), "%d", (++item_id));
-	struct oval_sysdata *sysdata = oval_sysdata_new(model, itemid);
-	oval_sysdata_set_status(sysdata, SYSCHAR_STATUS_NOT_COLLECTED);
-	oval_sysdata_set_subtype(sysdata, oval_object_get_subtype(object));
-	oval_syschar_add_sysdata(syschar, sysdata);
-
-	return syschar;
-}
-
-struct oval_sysinfo *oval_probe_sysinf_eval(struct oval_syschar_model *model, oval_pctx_t * pctx)
-{
-	return (NULL);
-a}
-
-void oval_pctx_free(oval_pctx_t * ctx)
-{
-}
-
-oval_pctx_t *oval_pctx_new(struct oval_syschar_model *sys)
-{
-	return (NULL);
-}
-#endif
