@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,9 +34,9 @@
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
 
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -107,7 +107,10 @@ struct oval_behavior *oval_behavior_clone(struct oval_definition_model *new_mode
 
 bool oval_behavior_is_valid(struct oval_behavior * behavior)
 {
-	return true;		//TODO
+	if (behavior == NULL)
+		return false;
+
+	return true;
 }
 
 bool oval_behavior_is_locked(struct oval_behavior * behavior)
