@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,19 +25,21 @@
  *
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
+ *      "Peter Vrabec" <pvabec@redhat.com>
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include "config.h"
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -1025,7 +1027,7 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 	int return_code = 0;
 
 	struct oval_definition_model *model = context->definition_model;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	/* oval_component_type_enum type; */
 	struct oval_component *component = NULL;
 	if (strcmp(tagname, "literal_component") == 0) {
@@ -1309,18 +1311,19 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(struct
 									 struct oval_component *component,
 									 struct oval_collection *value_collection)
 {
+#ifdef ENABLE_PROBES
 	__attribute__nonnull__(component);
 
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	struct oval_component_OBJECTREF *objref = (struct oval_component_OBJECTREF *)component;
 	struct oval_object *object = objref->object;
 	if (object) {
-		oval_pctx_t *pctx;
+		oval_probe_session_t *sess;
 		struct oval_syschar *syschar;
 
-		pctx = oval_pctx_new(sysmod);
-		syschar = oval_probe_object_eval(pctx, object);
-		oval_pctx_free(pctx);
+		sess = oval_probe_session_new(sysmod);
+		syschar = oval_probe_object_eval(sess, object, 0);
+		oval_probe_session_destroy(sess);
 
 		if (syschar) {
 			flag = oval_syschar_get_flag(syschar);
@@ -1345,6 +1348,11 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(struct
 		}
 	}
 	return flag;
+#else
+	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");	
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
+	return flag;
+#endif
 }
 
 #define _HAS_VALUES(flag) (flag==SYSCHAR_FLAG_COMPLETE || flag==SYSCHAR_FLAG_INCOMPLETE)
