@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
  *
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
+ *      "Peter Vrabec" <pvrabec@redhat.com>
  */
 
 #include <stdlib.h>
@@ -33,9 +34,9 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 /***************************************************************************/
 /* Variable definitions
@@ -204,7 +205,56 @@ struct oval_criteria_node *oval_criteria_node_new(struct oval_definition_model *
 
 bool oval_criteria_node_is_valid(struct oval_criteria_node * criteria_node)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	oval_criteria_node_type_t type;
+
+	if (criteria_node == NULL)
+		return false;
+
+	type = oval_criteria_node_get_type(criteria_node);
+	switch (type) {
+	case OVAL_NODETYPE_CRITERIA:
+		{
+			struct oval_criteria_node_iterator *subnodes_itr;
+
+			subnodes_itr = oval_criteria_node_get_subnodes(criteria_node);
+			while (oval_criteria_node_iterator_has_more(subnodes_itr)) {
+				struct oval_criteria_node *subnode;
+
+				subnode = oval_criteria_node_iterator_next(subnodes_itr);
+				if (oval_criteria_node_is_valid(subnode) != true) {
+					is_valid = false;
+					break;
+				}
+			}
+			oval_criteria_node_iterator_free(subnodes_itr);
+			if (is_valid != true)
+				return false;
+		}
+		break;
+	case OVAL_NODETYPE_CRITERION:
+		{
+			struct oval_test *test;
+
+			test = oval_criteria_node_get_test(criteria_node);
+			if (oval_test_is_valid(test) != true)
+				return false;
+		}
+		break;
+	case OVAL_NODETYPE_EXTENDDEF:
+		{
+			struct oval_definition *def;
+
+			def = oval_criteria_node_get_definition(criteria_node);
+			if (oval_definition_is_valid(def) != true)
+				return false;
+		}
+		break;
+	default:
+		return false;
+	}
+
+	return true;
 }
 
 bool oval_criteria_node_is_locked(struct oval_criteria_node * criteria_node)
@@ -378,7 +428,7 @@ static int _oval_criteria_subnode_consumer(xmlTextReaderPtr reader, struct oval_
 int oval_criteria_parse_tag(xmlTextReaderPtr reader,
 			    struct oval_parser_context *context, oval_criteria_consumer consumer, void *user)
 {
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	oval_criteria_node_type_t type = OVAL_NODETYPE_UNKNOWN;
 	if (strcmp(tagname, "criteria") == 0)
