@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,9 +34,9 @@
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 #define OVAL_SUPPORTED "5.5"
 
@@ -169,7 +169,16 @@ struct oval_definition *oval_definition_new(struct oval_definition_model *model,
 
 bool oval_definition_is_valid(struct oval_definition * definition)
 {
-	return true;		//TODO
+	struct oval_criteria_node *criteria_node;
+
+	if (definition == NULL)
+		return false;
+
+	criteria_node = oval_definition_get_criteria(definition);
+	if (oval_criteria_node_is_valid(criteria_node) != true)
+		return false;
+
+	return true;
 }
 
 bool oval_definition_is_locked(struct oval_definition * definition)
@@ -419,7 +428,7 @@ static void oval_reference_consume(struct oval_reference *ref, void *def)
 static int _oval_definition_parse_metadata(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	struct oval_definition *definition = (struct oval_definition *)user;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	int return_code;
 	if ((strcmp(tagname, "title") == 0)) {
 		return_code = oval_parser_text_value(reader, context, &_oval_definition_title_consumer, definition);
@@ -456,7 +465,7 @@ static void _oval_definition_criteria_consumer(struct oval_criteria_node *criter
 static int _oval_definition_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	struct oval_definition *definition = (struct oval_definition *)user;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	int return_code;
 	if ((strcmp(tagname, "metadata") == 0)) {
 		return_code = oval_parser_parse_tag(reader, context, &_oval_definition_parse_metadata, definition);
