@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -32,8 +32,8 @@
 #include "oval_string_map_impl.h"
 #include "oval_system_characteristics_impl.h"
 #include "oval_results_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 #define OVAL_ENUMERATION_INVALID (-1)
 
