@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,7 +33,7 @@
 #include "oval_agent_api_impl.h"
 #include "oval_system_characteristics_impl.h"
 #include "oval_collection_impl.h"
-#include "../common/public/debug.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_message {
 	void *model;
