@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,7 +33,7 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/public/debug.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_object {
 	struct oval_definition_model *model;
@@ -161,7 +161,44 @@ struct oval_object *oval_object_new(struct oval_definition_model *model, char *i
 
 bool oval_object_is_valid(struct oval_object * object)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_object_content_iterator *contents_itr;
+	struct oval_behavior_iterator *behaviors_itr;
+
+	if (object == NULL)
+		return false;
+
+	/* validate object contents */
+	contents_itr = oval_object_get_object_contents(object);
+	while (oval_object_content_iterator_has_more(contents_itr)) {
+		struct oval_object_content *content;
+
+		content = oval_object_content_iterator_next(contents_itr);
+		if (oval_object_content_is_valid(content) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_object_content_iterator_free(contents_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate behaviors */
+	behaviors_itr = oval_object_get_behaviors(object);
+	while (oval_behavior_iterator_has_more(behaviors_itr)) {
+		struct oval_behavior *behavior;
+
+		behavior = oval_behavior_iterator_next(behaviors_itr);
+		if (oval_behavior_is_valid(behavior) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_behavior_iterator_free(behaviors_itr);
+	if (is_valid != true)
+		return false;
+
+	return true;
 }
 
 bool oval_object_is_locked(struct oval_object * object)
@@ -310,7 +347,7 @@ static void oval_content_consume(struct oval_object_content *content, void *obje
 static int _oval_object_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	struct oval_object *object = (struct oval_object *)user;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	int return_code = 1;
 	if ((strcmp(tagname, "notes") == 0)) {
