@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_object_content {
 	struct oval_definition_model *model;
@@ -169,7 +169,36 @@ struct oval_object_content
 
 bool oval_object_content_is_valid(struct oval_object_content * object_content)
 {
-	return true;		//TODO
+	oval_object_content_type_t type;
+
+	if (object_content == NULL)
+		return false;
+
+	type = oval_object_content_get_type(object_content);
+	switch (type) {
+	case OVAL_OBJECTCONTENT_ENTITY:
+		{
+			struct oval_entity *entity;
+
+			entity = oval_object_content_get_entity(object_content);
+			if (oval_entity_is_valid(entity) != true)
+				return false;
+		}
+		break;
+	case OVAL_OBJECTCONTENT_SET:
+		{
+			struct oval_setobject *set;
+
+			set = oval_object_content_get_setobject(object_content);
+			if (oval_setobject_is_valid(set) != true)
+				return false;
+		}
+		break;
+	default:
+		return false;
+	}
+
+	return true;
 }
 
 bool oval_object_content_is_locked(struct oval_object_content * object_content)
@@ -186,7 +215,7 @@ struct oval_object_content *oval_object_content_clone
 	oval_object_content_set_field_name(new_content, name);
 	switch (new_content->type) {
 	case OVAL_OBJECTCONTENT_ENTITY:{
-			struct oval_entity *entity = oval_object_content_get_entity(old_content);
+			struct oval_entity *entity = oval_entity_clone(new_model, oval_object_content_get_entity(old_content));
 			oval_object_content_set_entity(new_content, entity);
 			oval_check_t check = oval_object_content_get_varCheck(old_content);
 			oval_object_content_set_varCheck(new_content, check);
@@ -303,7 +332,7 @@ int oval_object_content_parse_tag(xmlTextReaderPtr reader,
 {
 	__attribute__nonnull__(context);
 
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 
 	oval_object_content_type_t type =
