@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include <string.h>
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_result_criteria_node {
 	struct oval_result_system *sys;
@@ -126,7 +126,59 @@ struct oval_result_criteria_node *oval_result_criteria_node_new
 
 bool oval_result_criteria_node_is_valid(struct oval_result_criteria_node * result_criteria_node)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	oval_criteria_node_type_t type;
+
+	if (result_criteria_node == NULL)
+		return false;
+
+	type = oval_result_criteria_node_get_type(result_criteria_node);
+	switch (type) {
+	case OVAL_NODETYPE_CRITERIA:
+		{
+			struct oval_result_criteria_node_iterator *subnodes_itr;
+
+			/* validate subnodes */
+			subnodes_itr = oval_result_criteria_node_get_subnodes(result_criteria_node);
+			while (oval_result_criteria_node_iterator_has_more(subnodes_itr)) {
+				struct oval_result_criteria_node *subnode;
+
+				subnode = oval_result_criteria_node_iterator_next(subnodes_itr);
+				if (oval_result_criteria_node_is_valid(subnode) != true) {
+					is_valid = false;
+					break;
+				}
+			}
+			oval_result_criteria_node_iterator_free(subnodes_itr);
+			if (is_valid != true)
+				return false;
+		}
+		break;
+	case OVAL_NODETYPE_CRITERION:
+		{
+			struct oval_result_test *rslt_test;
+
+			/* validate test */
+			rslt_test = oval_result_criteria_node_get_test(result_criteria_node);
+			if (oval_result_test_is_valid(rslt_test) != true)
+				return false;
+		}
+		break;
+	case OVAL_NODETYPE_EXTENDDEF:
+		{
+			struct oval_result_definition *rslt_def;
+
+			/* validate definition */
+			rslt_def = oval_result_criteria_node_get_extends(result_criteria_node);
+			if (oval_result_definition_is_valid(rslt_def) != true)
+				return false;
+		}
+		break;
+	default:
+		return false;
+	}
+
+	return true;
 }
 
 bool oval_result_criteria_node_is_locked(struct oval_result_criteria_node * result_criteria_node)
@@ -382,7 +434,10 @@ static oval_result_t _oval_result_criteria_node_result(struct oval_result_criter
 	default:
 		break;
 	}
-	result = (result == OVAL_RESULT_INVALID) ? OVAL_RESULT_UNKNOWN : _oval_result_negate(node->negate, result);
+
+	if ( result != OVAL_RESULT_INVALID ) 
+		result =  _oval_result_negate(node->negate, result);
+
 	return result;
 }
 
