@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "public/oval_agent_api.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_result_definition {
 	struct oval_definition *definition;
@@ -81,7 +81,26 @@ struct oval_result_definition *oval_result_definition_new(struct oval_result_sys
 
 bool oval_result_definition_is_valid(struct oval_result_definition * result_definition)
 {
-	return true;		//TODO
+	struct oval_definition *definition;
+	struct oval_result_criteria_node *rslt_criteria_node;
+
+	if (result_definition == NULL)
+		return false;
+
+	if (oval_result_definition_get_system(result_definition) == NULL)
+		return false;
+
+	/* validate definition */
+	definition = oval_result_definition_get_definition(result_definition);
+	if (oval_definition_is_valid(definition) != true)
+		return false;
+
+	/* validate criteria */
+	rslt_criteria_node = oval_result_definition_get_criteria(result_definition);
+	if (oval_result_criteria_node_is_valid(rslt_criteria_node) != true)
+		return false;
+
+	return true;
 }
 
 bool oval_result_definition_is_locked(struct oval_result_definition * result_definition)
