@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,18 +25,20 @@
  *
  * Authors:
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
+ *      "Peter Vrabec" <pvrabec@redhat.com>
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include "oval_definitions.h"
+#include "oval_agent_api.h"
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
-#include "public/oval_definitions.h"
-#include "public/oval_agent_api.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
+#include "common/util.h"
 
 typedef struct oval_result_system {
 	struct oval_results_model *model;
@@ -63,7 +65,50 @@ struct oval_result_system *oval_result_system_new(struct oval_results_model *mod
 
 bool oval_result_system_is_valid(struct oval_result_system * result_system)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_result_definition_iterator *rslt_definitions_itr;
+	struct oval_result_test_iterator *rslt_tests_itr;
+	struct oval_syschar_model *syschar_model;
+
+	if (result_system == NULL)
+		return false;
+
+	/* validate syschar_model */
+	syschar_model = oval_result_system_get_syschar_model(result_system);
+	if (oval_syschar_model_is_valid(syschar_model) != true)
+		return false;
+
+	/* validate result definitions */
+	rslt_definitions_itr = oval_result_system_get_definitions(result_system);
+	while (oval_result_definition_iterator_has_more(rslt_definitions_itr)) {
+		struct oval_result_definition *rslt_definition;
+
+		rslt_definition = oval_result_definition_iterator_next(rslt_definitions_itr);
+		if (oval_result_definition_is_valid(rslt_definition) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_result_definition_iterator_free(rslt_definitions_itr);
+	if (is_valid != true)
+		return false;
+
+	/* validate result tests */
+	rslt_tests_itr = oval_result_system_get_tests(result_system);
+	while (oval_result_test_iterator_has_more(rslt_tests_itr)) {
+		struct oval_result_test *rslt_test;
+
+		rslt_test = oval_result_test_iterator_next(rslt_tests_itr);
+		if (oval_result_test_is_valid(rslt_test) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_result_test_iterator_free(rslt_tests_itr);
+	if (is_valid != true)
+		return false;
+
+	return true;
 }
 
 bool oval_result_system_is_locked(struct oval_result_system * result_system)
@@ -492,11 +537,12 @@ static bool _oval_result_system_resolve_syschar(struct oval_syschar *syschar, st
 	return oval_string_map_get_value(sysmap, objid) != NULL;
 }
 
-void oval_result_system_eval(struct oval_result_system *sys)
+int oval_result_system_eval(struct oval_result_system *sys)
 {
 	struct oval_results_model *res_model;
 	struct oval_definition_model *definition_model;
 	struct oval_definition_iterator *definitions_itr;
+	oval_result_t result;
 
 	res_model = oval_result_system_get_results_model(sys);
 	definition_model = oval_results_model_get_definition_model(res_model);
@@ -508,12 +554,43 @@ void oval_result_system_eval(struct oval_result_system *sys)
 
 		definition = oval_definition_iterator_next(definitions_itr);
 		rslt_definition = oval_result_system_get_new_definition(sys, definition);
-		oval_result_definition_eval(rslt_definition);
+		result = oval_result_definition_eval(rslt_definition);
+		if( result == OVAL_RESULT_INVALID ) {
+			oval_definition_iterator_free(definitions_itr);
+			return -1;
+		}
 	}
 
 	oval_definition_iterator_free(definitions_itr);
+	return 0;
 }
 
+oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys, char *id)
+{
+        struct oval_results_model *res_model;
+        struct oval_definition_model *definition_model;
+        struct oval_definition *oval_definition;
+	struct oval_result_definition *rslt_definition;
+
+	res_model = oval_result_system_get_results_model(sys);
+	definition_model = oval_results_model_get_definition_model(res_model);
+	oval_definition = oval_definition_model_get_definition(definition_model, id);
+	if (oval_definition == NULL) {
+		oscap_dprintf("%s:%d No definition with ID (%s) in definition model.", __FILE__, __LINE__, id);
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "No definition with such an ID in definition model.");
+		return OVAL_RESULT_INVALID;
+	}
+ 
+        rslt_definition = oval_result_system_get_definition(sys, id);
+        if (rslt_definition == NULL) {
+        	rslt_definition = make_result_definition_from_oval_definition(sys, oval_definition);
+                oval_result_system_add_definition(sys, rslt_definition);
+        }
+
+	return oval_result_definition_eval(rslt_definition);
+}
+
+
 xmlNode *oval_result_system_to_dom
     (struct oval_result_system * sys,
      struct oval_results_model * results_model,
