@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -154,7 +154,29 @@ struct oval_state *oval_state_new(struct oval_definition_model *model, char *id)
 
 bool oval_state_is_valid(struct oval_state * state)
 {
-	return true;		//TODO
+	/*
+	bool is_valid = true;
+	struct oval_state_content_iterator *contents_itr;
+	*/
+
+	if (state == NULL)
+		return false;
+	/*
+	contents_itr = oval_state_get_contents(state);
+	while (oval_state_content_iterator_has_more(contents_itr)) {
+		struct oval_state_content *content;
+
+		content = oval_state_content_iterator_next(contents_itr);
+		if (oval_state_content_is_valid(content) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_state_content_iterator_free(contents_itr);
+	if (is_valid != true)
+		return false;
+	*/
+	return true;
 }
 
 bool oval_state_is_locked(struct oval_state * state)
@@ -289,7 +311,7 @@ static int _oval_state_parse_tag(xmlTextReaderPtr reader, struct oval_parser_con
 	__attribute__nonnull__(user);
 
 	struct oval_state *state = (struct oval_state *)user;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	int return_code = 1;
 	if ((strcmp(tagname, "notes") == 0)) {
