@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include "oval_system_characteristics_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_syschar {
 	struct oval_syschar_model *model;
@@ -163,7 +163,37 @@ struct oval_syschar *oval_syschar_new(struct oval_syschar_model *model, struct o
 
 bool oval_syschar_is_valid(struct oval_syschar * syschar)
 {
-	return true;		//TODO
+	bool is_valid = true;
+	struct oval_object *object;
+	struct oval_sysdata_iterator *sysdatas_itr;
+
+	if (syschar == NULL)
+		return false;
+
+	/* validate object */
+	object = oval_syschar_get_object(syschar);
+	if (oval_object_is_valid(object) != true)
+		return false;
+
+	/* validate variable_bindings */
+	// todo
+
+	/* validate sysdatas */
+	sysdatas_itr = oval_syschar_get_sysdata(syschar);
+	while (oval_sysdata_iterator_has_more(sysdatas_itr)) {
+		struct oval_sysdata *sysdata;
+
+		sysdata = oval_sysdata_iterator_next(sysdatas_itr);
+		if (oval_sysdata_is_valid(sysdata) != true) {
+			is_valid = false;
+			break;
+		}
+	}
+	oval_sysdata_iterator_free(sysdatas_itr);
+	if (is_valid != true)
+		return false;
+
+	return true;
 }
 
 bool oval_syschar_is_locked(struct oval_syschar * syschar)
@@ -295,7 +325,7 @@ int oval_syschar_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *
 {
 	__attribute__nonnull__(context);
 
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
 	oscap_dprintf("DEBUG: oval_syschar_parse_tag(<%s:%s>): enter", namespace, tagname);
 	int is_ovalsys = strcmp(namespace, NAMESPACE_OVALSYS) == 0;
