@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_test {
 	struct oval_definition_model *model;
@@ -166,7 +166,24 @@ struct oval_test *oval_test_new(struct oval_definition_model *model, char *id)
 
 bool oval_test_is_valid(struct oval_test * test)
 {
-	return true;		//TODO
+	struct oval_object *object;
+	struct oval_state *state;
+
+	if (test == NULL)
+		return false;
+
+	if (oval_test_get_subtype(test) == OVAL_INDEPENDENT_UNKNOWN)
+		return true;
+
+	object = oval_test_get_object(test);
+	if (oval_object_is_valid(object) != true)
+		return false;
+
+	state = oval_test_get_state(test);
+	if (state != NULL && oval_state_is_valid(state) != true)
+		return false;
+
+	return true;
 }
 
 bool oval_test_is_locked(struct oval_test * test)
@@ -317,7 +334,7 @@ static int _oval_test_parse_notes(xmlTextReaderPtr reader, struct oval_parser_co
 static int _oval_test_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	struct oval_test *test = (struct oval_test *)user;
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	//xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	int return_code = 1;
 	if ((strcmp(tagname, "notes") == 0)) {
