@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -33,8 +33,8 @@
 #include <stdbool.h>
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
 
 typedef struct oval_value {
 	oval_datatype_t datatype;
@@ -181,12 +181,11 @@ int oval_value_parse_tag(xmlTextReaderPtr reader,
 	char *text = NULL;
 	int isNil = oval_parser_boolean_attribute(reader, "xsi:nil", 0);
 	if (isNil) {
-		text = NULL;
 		return_code = 1;
 	} else {
 		return_code = oval_parser_text_value(reader, context, &oval_value_parse_tag_consume_text, &text);
 	}
-	struct oval_value *value = oval_value_new(datatype, text);
+	struct oval_value *value = oval_value_new(datatype, text ? text : "");
 	oscap_free(text);
 	(*consumer) (value, user);
 	return return_code;
