@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,9 +34,9 @@
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
-#include "../common/public/debug.h"
-#include "../common/_error.h"
+#include "common/util.h"
+#include "common/debug_priv.h"
+#include "common/_error.h"
 
 typedef struct oval_variable {
 	struct oval_definition_model *model;
@@ -229,7 +229,7 @@ struct oval_variable *oval_variable_new(struct oval_definition_model *model, cha
 		break;
 	default:
 		oscap_dprintf(" oval_variable type not valid: type = %d (%s:%d)", type, __FILE__, __LINE__);
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EVARTYPE, "Invalid OVAL variable type");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Invalid OVAL variable type");
 		return NULL;
 	}
 
@@ -407,7 +407,7 @@ static void _oval_variable_parse_local_tag_component_consumer(struct oval_compon
 static int _oval_variable_parse_local_tag(xmlTextReaderPtr reader, struct oval_parser_context *context, void *user)
 {
 	struct oval_variable *variable = (struct oval_variable *)user;
-	xmlChar *tagname = xmlTextReaderName(reader);
+	xmlChar *tagname = xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	int return_code = oval_component_parse_tag(reader, context, &_oval_variable_parse_local_tag_component_consumer,
 						   variable);
@@ -434,7 +434,7 @@ static void _oval_variable_parse_value(char *text_value, struct oval_variable *v
 static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader,
 					     struct oval_parser_context *context, struct oval_variable *variable)
 {
-	xmlChar *tagname = xmlTextReaderName(reader);
+	xmlChar *tagname = xmlTextReaderLocalName(reader);
 	xmlChar *namespace = xmlTextReaderNamespaceUri(reader);
 	int return_code = 1;
 	if (strcmp("value", (char *)tagname) == 0 && strcmp(DEFINITION_NAMESPACE, (char *)namespace) == 0) {
@@ -453,7 +453,7 @@ static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader,
 int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *context)
 {
 	struct oval_definition_model *model = oval_parser_context_model(context);
-	char *tagname = (char *)xmlTextReaderName(reader);
+	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	oval_variable_type_t type;
 	if (strcmp(tagname, "constant_variable") == 0)
 		type = OVAL_VARIABLE_CONSTANT;
