@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,6 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,8 +43,8 @@ static int SEXP_sbprintf_t_cb (SEXP_t *s_exp, strbuf_t *sb)
 {
         if (SEXP_sbprintf_t (s_exp, sb) != 0)
                 return (-1);
-        if (strbuf_add (sb, " ", 1) != 0)
-                return (-1);
+        /*if (strbuf_add (sb, " ", 1) != 0)
+          return (-1); */
         else
                 return (0);
 }
@@ -54,31 +53,33 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
 {
         SEXP_val_t v_dsc;
         int buflen;
-        
+
         if (s_exp->s_type != NULL) {
-                char buffer[64+1];
-                
-                buflen = snprintf (buffer, sizeof buffer, "%zu[%s]",
-                                   strlen (s_exp->s_type->name), s_exp->s_type->name);
-                
+                const char *name;
+                char  buffer[64+1];
+
+                name   = SEXP_datatype_name(s_exp->s_type);
+                buflen = snprintf (buffer, sizeof buffer,
+                                   "#d%zu[%s]", strlen (name), name);
+
                 _A((size_t)buflen < sizeof buffer);
 
                 if (strbuf_add (sb, buffer, buflen) != 0)
                         return (-1);
         }
-        
+
         SEXP_val_dsc (&v_dsc, s_exp->s_valp);
-        
+
         switch (v_dsc.type) {
         case SEXP_VALTYPE_NUMBER:
         {
                 SEXP_numtype_t t;
-                
+
                 t = SEXP_NTYPEP(v_dsc.hdr->size, v_dsc.mem);
-                
+
                 if (t <= SEXP_NUM_UINT16) {
                         if (t <= SEXP_NUM_UINT8) {
-                                char buffer[4+1];
+                                char buffer[6+1];
 
                                 switch (t) {
                                 case SEXP_NUM_BOOL:
@@ -87,11 +88,11 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
                                         break;
                                 case SEXP_NUM_INT8:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%hhd", SEXP_NCASTP(i8,v_dsc.mem)->n);
+                                                           "#d%hhd", SEXP_NCASTP(i8,v_dsc.mem)->n);
                                         break;
                                 case SEXP_NUM_UINT8:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%hhu", SEXP_NCASTP(u8,v_dsc.mem)->n);
+                                                           "#d%hhu", SEXP_NCASTP(u8,v_dsc.mem)->n);
                                         break;
                                 default:
                                         abort ();
@@ -99,21 +100,21 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
 
                                 _A(buflen >= 0);
                                 _A((size_t)buflen < sizeof buffer);
-                                
+
                                 if (strbuf_add (sb, (const char *)buffer, (size_t)buflen) != 0)
                                         return (-1);
-                                
+
                         } else {
-                                char buffer[6+1];
-                                
+                                char buffer[8+1];
+
                                 switch (t) {
                                 case SEXP_NUM_INT16:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%hd", SEXP_NCASTP(i16,v_dsc.mem)->n);
+                                                           "#d%hd", SEXP_NCASTP(i16,v_dsc.mem)->n);
                                         break;
                                 case SEXP_NUM_UINT16:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%hu", SEXP_NCASTP(u16,v_dsc.mem)->n);
+                                                           "#d%hu", SEXP_NCASTP(u16,v_dsc.mem)->n);
                                         break;
                                 default:
                                         abort ();
@@ -121,27 +122,27 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
 
                                 _A(buflen >= 0);
                                 _A((size_t)buflen < sizeof buffer);
-                                
+
                                 if (strbuf_add (sb, (const char *)buffer, (size_t)buflen) != 0)
                                         return (-1);
-                                
+
                         }
                 } else {
                         if (t <= SEXP_NUM_INT64) {
-                                char buffer[20+1];
-                                
+                                char buffer[22+1];
+
                                 switch (t) {
                                 case SEXP_NUM_INT32:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%" PRId32, SEXP_NCASTP(i32,v_dsc.mem)->n);
+                                                           "#d%" PRId32, SEXP_NCASTP(i32,v_dsc.mem)->n);
                                         break;
                                 case SEXP_NUM_UINT32:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%" PRIu32, SEXP_NCASTP(u32,v_dsc.mem)->n);
+                                                           "#d%" PRIu32, SEXP_NCASTP(u32,v_dsc.mem)->n);
                                         break;
                                 case SEXP_NUM_INT64:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%" PRId64, SEXP_NCASTP(i64,v_dsc.mem)->n);
+                                                           "#d%" PRId64, SEXP_NCASTP(i64,v_dsc.mem)->n);
                                         break;
                                 default:
                                         abort ();
@@ -149,21 +150,21 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
 
                                 _A(buflen >= 0);
                                 _A((size_t)buflen < sizeof buffer);
-                                
+
                                 if (strbuf_add (sb, (const char *)buffer, (size_t)buflen) != 0)
                                         return (-1);
-                                
+
                         } else {
-                                char buffer[64+1];
-                                
+                                char buffer[66+1];
+
                                 switch (t) {
                                 case SEXP_NUM_UINT64:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%" PRIu64, SEXP_NCASTP(u64,v_dsc.mem)->n);
+                                                           "#d%" PRIu64, SEXP_NCASTP(u64,v_dsc.mem)->n);
                                         break;
                                 case SEXP_NUM_DOUBLE:
                                         buflen = snprintf (buffer, sizeof buffer,
-                                                           "%f", SEXP_NCASTP(f  ,v_dsc.mem)->n);
+                                                           "#d%g", SEXP_NCASTP(f  ,v_dsc.mem)->n);
                                         break;
                                 default:
                                         abort ();
@@ -171,10 +172,10 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
 
                                 _A(buflen >= 0);
                                 _A((size_t)buflen < sizeof buffer);
-                                
+
                                 if (strbuf_add (sb, (const char *)buffer, (size_t)buflen) != 0)
                                         return (-1);
-                                
+
                         }
                 }
                 break;
@@ -182,36 +183,36 @@ int SEXP_sbprintf_t (SEXP_t *s_exp, strbuf_t *sb)
         case SEXP_VALTYPE_STRING:
         {
                 char buffer[20+1];
-                
-                buflen = snprintf (buffer, sizeof buffer, "%zu:", v_dsc.hdr->size / sizeof (char));
-                
+
+                buflen = snprintf (buffer, sizeof buffer, "#d%zu:", v_dsc.hdr->size / sizeof (char));
+
                 _A(buflen >= 0);
                 _A((size_t)buflen < sizeof buffer);
-                
+
                 if (strbuf_add (sb, (const char *)buffer, (size_t)buflen) != 0)
                         return (-1);
                 if (strbuf_add (sb, (const char *)v_dsc.mem, v_dsc.hdr->size / sizeof (char)) != 0)
                         return (-1);
-                
+
                 break;
         }
         case SEXP_VALTYPE_LIST:
-                
+
                 if (strbuf_add (sb, "(", 1) != 0)
                         return (-1);
-                if (SEXP_rawval_lblk_cb ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr, (int (*)(SEXP_t *, void *)) SEXP_sbprintf_t_cb, (void *)sb,
+                if (SEXP_rawval_lblk_cb ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr, (int (*)(SEXP_t *, void *)) SEXP_sbprintf_t, (void *)sb,
                                          SEXP_LCASTP(v_dsc.mem)->offset + 1) != 0)
                         return (-1);
                 if (strbuf_trunc (sb, 1) != 0)
                         return (-1);
                 if (strbuf_add (sb, ")", 1) != 0)
                         return (-1);
-                
+
                 break;
         default:
                 abort ();
         }
-        
+
         return (0);
 }
 
@@ -232,19 +233,19 @@ static int __SEXP_fprintfa_lmemb (const SEXP_t *s_exp, void *arg)
 size_t SEXP_fprintfa (FILE *fp, const SEXP_t *s_exp)
 {
         SEXP_val_t v_dsc;
-        
+
         if (s_exp->s_type != NULL)
-                fprintf (fp, "[%s]", s_exp->s_type->name);
-        
+                fprintf (fp, "[%s]", SEXP_datatype_name(s_exp->s_type));
+
         SEXP_val_dsc (&v_dsc, s_exp->s_valp);
-        
+
         switch (v_dsc.type) {
         case SEXP_VALTYPE_NUMBER:
         {
                 SEXP_numtype_t t;
-                
+
                 t = SEXP_NTYPEP(v_dsc.hdr->size, v_dsc.mem);
-                
+
                 if (t <= SEXP_NUM_UINT16) {
                         if (t <= SEXP_NUM_UINT8) {
                                 switch (t) {
@@ -268,11 +269,11 @@ size_t SEXP_fprintfa (FILE *fp, const SEXP_t *s_exp)
                         } else {
                                 switch (t) {
                                 case SEXP_NUM_UINT64: return fprintf (fp, "%" PRIu64, SEXP_NCASTP(u64,v_dsc.mem)->n);
-                                case SEXP_NUM_DOUBLE: return fprintf (fp,   "%f", SEXP_NCASTP(f  ,v_dsc.mem)->n);
+                                case SEXP_NUM_DOUBLE: return fprintf (fp,   "%g", SEXP_NCASTP(f  ,v_dsc.mem)->n);
                                 }
                         }
                 }
-                
+
                 abort ();
         }       break;
         case SEXP_VALTYPE_STRING:
@@ -280,23 +281,23 @@ size_t SEXP_fprintfa (FILE *fp, const SEXP_t *s_exp)
         case SEXP_VALTYPE_LIST:
         {
                 __fprintfa_t fpa;
-                
+
                 fpa.sz = 2;
                 fpa.fp = fp;
                 fputc ('(', fp);
-                
+
                 SEXP_rawval_lblk_cb ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr,
                                      (int(*)(SEXP_t *, void *))__SEXP_fprintfa_lmemb, &fpa,
                                      SEXP_LCASTP(v_dsc.mem)->offset + 1);
-                
+
                 fputc (')', fp);
-         
+
                 return (fpa.sz);
         }
         default:
                 abort ();
         }
-        
+
         return (0);
 }
 
@@ -311,7 +312,7 @@ static SEXP_ostate_t *__SEXP_ostate_new (void)
         ost->sexp_pos = 0;
         ost->sexp_part = 0;
         LIST_stack_init (&(ost->lstack));
-        
+
         return (ost);
 }
 
@@ -379,7 +380,7 @@ int SEXP_printf (SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -396,7 +397,7 @@ int SEXP_fprintf (FILE *fp, SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -413,7 +414,7 @@ int SEXP_sprintf (char *str, SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -430,7 +431,7 @@ int SEXP_snprintf (char *str, size_t size, SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -447,7 +448,7 @@ int SEXP_asprintf (char **ret, SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -464,7 +465,7 @@ int SEXP_asnprintf (char **ret, size_t maxsz, SEXP_format_t fmt, SEXP_t *sexp)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -474,17 +475,17 @@ int SEXP_asnprintf (char **ret, size_t maxsz, SEXP_format_t fmt, SEXP_t *sexp)
 size_t SEXP_st_dprintc (int fd, SEXP_t *s_exp, SEXP_ostate_t **ost)
 {
         SEXP_t *s_cur;
-        
+
         /* create output state */
-        
+
 
         for (;;) {
                 s_cur = SEXP_ostate_sexp (*ost);
-                
+
                 if (s_cur->s_type != NULL) {
                         /* print datatype */
                 }
-                 
+
                 switch (SEXP_typeof (
         }
 }
@@ -498,15 +499,15 @@ ssize_t SEXP_st_dprintc (int fd, SEXP_t *sexp, SEXP_ostate_t **ost)
         uint32_t digits;
         ssize_t wret = -1;
         size_t  wlen =  0;
-        
+
         struct iovec vec[3];
-        
+
         _A(fd >= 0);
         _A(sexp != NULL);
         _A(ost  != NULL);
 
 #define SCNT (LIST_stack_cnt (&((*ost)->lstack)))
-        
+
         if (*ost == NULL) {
                 *ost = __SEXP_ostate_new ();
                 (*ost)->sexp = psexp = sexp;
@@ -514,18 +515,18 @@ ssize_t SEXP_st_dprintc (int fd, SEXP_t *sexp, SEXP_ostate_t **ost)
                 psexp = (*ost)->sexp;
                 /* FIXME */
         }
-        
+
 print_loop:
         if (psexp->handler != NULL) {
                 SEXP_datatype_t *h = psexp->handler;
-                
+
                 ch = ']';
-                
+
                 digits = xnumdigits (h->name_len);
                 buffer = sm_alloc (sizeof (char) * (digits + 2));
                 snprintf (buffer, sizeof (char) * (digits + 2), "%.*hu[",
                           digits, h->name_len);
-                
+
                 vec[0].iov_base = buffer;
                 vec[0].iov_len  = digits + 1;
                 vec[1].iov_base = h->name;
@@ -534,22 +535,22 @@ print_loop:
                 vec[2].iov_len  = 1;
 
                 wret = writev (fd, vec, 3);
-                
+
                 _D("wrote: %.*s%.*s\n", digits + 1, buffer, h->name_len, h->name);
-                
+
                 if (wret == -1) {
                         protect_errno {
                                 _D("errno=%u, %s.\n", errno, strerror (errno));
                         }
-                        
+
                         return (-1);
                 }
-                
+
                 sm_free (buffer);
-                
+
                 wlen += (size_t)wret;
         }
-        
+
         switch (SEXP_TYPE(psexp)) {
         case ATOM_LIST:
                 ch = '(';
@@ -558,20 +559,20 @@ print_loop:
                         protect_errno {
                                 _D("errno=%u, %s.\n", errno, strerror (errno));
                         }
-                
+
                         return (-1);
                 }
-                
+
                 _D("wrote: (\n");
-                
+
                 wlen += sizeof ch;
-                
+
                 LIST_stack_push (&((*ost)->lstack), &(psexp->atom.list));
                 (*ost)->list_pos = sm_realloc ((*ost)->list_pos,
                                                sizeof (uint32_t) * SCNT);
-                
+
                 _A(SCNT > 0);
-                
+
                 (*ost)->list_pos[SCNT - 1] = 0;
                 break;
         case ATOM_STRING:
@@ -579,26 +580,26 @@ print_loop:
                 buffer = sm_alloc (sizeof (char) * (digits + 2));
                 snprintf (buffer, sizeof (char) * (digits + 2), "%.*u:",
                           digits, psexp->atom.string.len);
-                
+
                 vec[0].iov_base = buffer;
                 vec[0].iov_len  = digits + 1;
                 vec[1].iov_base = psexp->atom.string.str;
                 vec[1].iov_len  = psexp->atom.string.len;
-                
+
                 _D("wrote: %.*s%.*s\n", digits + 1, buffer,
                    psexp->atom.string.len, psexp->atom.string.str);
-                
+
                 if ((wret = writev (fd, vec, 2)) == -1) {
                         /* write error */
                         protect_errno {
                                 _D("errno=%u, %s.\n", errno, strerror (errno));
                         }
-                
+
                         return (-1);
                 }
-                
+
                 sm_free (buffer);
-                
+
                 if ((wret - digits - 1 - psexp->atom.string.len) != 0) {
                         /* not everything was written to fd */
                         (*ost)->sexp_pos  = wret;
@@ -607,16 +608,16 @@ print_loop:
                         protect_errno {
                                 _D("errno=%u, %s.\n", errno, strerror (errno));
                         }
-                
+
                         return (-1);
                 }
-                
+
                 wlen += (size_t)wret;
                 break;
         case ATOM_NUMBER: {
                 char numstr[64];
                 int  numlen;
-                /* 
+                /*
                  * Number is converted to string with datatype.
                  * e.g.: 123 -> 4[int8]3:123
                  *
@@ -630,7 +631,7 @@ print_loop:
                                            __fmt, num);                 \
                 }                                                       \
                         break
-                
+
                 switch (psexp->atom.number.type) {
                         CASE(NUM_INT8,     int8_t, SEXP_number_gethhd, "%hhd ");
                         CASE(NUM_UINT8,   uint8_t, SEXP_number_gethhu, "%hhu ");
@@ -650,28 +651,28 @@ print_loop:
                         protect_errno {
                                 _D("errno=%u, %s.\n", errno, strerror (errno));
                         }
-                        
+
                         return (-1);
                 }
-                
+
                 _D("wrote: %.*s\n", numlen, numstr);
 
                 wlen += numlen;
         }
-                break;  
+                break;
         default:
                 abort ();
         }
 
         while (SCNT > 0) {
                 LIST_t *ltop;
-                
+
                 ltop = LIST_stack_top (&((*ost)->lstack));
-                
+
                 if ((*ost)->list_pos[SCNT - 1] < ltop->count) {
                         psexp = &(SEXP(ltop->memb)[((*ost)->list_pos[SCNT - 1])]);
                         ++((*ost)->list_pos[SCNT - 1]);
-                        
+
                         goto print_loop;
                 } else {
                         /* end of list */
@@ -684,25 +685,25 @@ print_loop:
                                 protect_errno {
                                         _D("errno=%u, %s.\n", errno, strerror (errno));
                                 }
-                
+
                                 return (-1);
                         }
-                        
+
                         wlen += sizeof ch;
-                        
+
                         LIST_stack_dec (&((*ost)->lstack));
                         (*ost)->list_pos = sm_realloc ((*ost)->list_pos,
                                                        sizeof (uint32_t) * SCNT);
                 }
         }
-        
-        /* 
+
+        /*
          *  If we got here, we can destroy
          *  the ostate because everything
          *  went fine.
          */
         (*ost) = NULL;
-        
+
         return ((ssize_t)wlen);
 }
 
@@ -723,7 +724,7 @@ int SEXP_fprintfc (FILE *fp, SEXP_t *sexp)
 
 int SEXP_dprintfc (int fd, SEXP_t *sexp)
 {
-        
+
         return (-1);
 }
 
@@ -766,12 +767,12 @@ int SEXP_printfa (SEXP_t *sexp)
 static int __SEXP_fprintfa (FILE *fp, SEXP_t *sexp, uint32_t indent)
 {
         const char *datatype;
-        
+
         datatype = SEXP_datatype (sexp);
-        
+
         if (datatype != NULL)
                 fprintf (fp, "[%s]", datatype);
-        
+
         switch (SEXP_TYPE(sexp)) {
         case ATOM_STRING:
                 return fprintf (fp, "\"%.*s\"", sexp->atom.string.len, sexp->atom.string.str);
@@ -802,7 +803,7 @@ static int __SEXP_fprintfa (FILE *fp, SEXP_t *sexp, uint32_t indent)
                 break;
         case ATOM_LIST: {
                 uint32_t i;
-                
+
                 if (indent > 0)
                         putc ('\n', fp);
                 for (i = 0; i < indent; ++i)
@@ -823,7 +824,7 @@ static int __SEXP_fprintfa (FILE *fp, SEXP_t *sexp, uint32_t indent)
                                 }
                         }
                 }
-                
+
                 putc (')', fp);
                 return (0);
         }
@@ -832,7 +833,7 @@ static int __SEXP_fprintfa (FILE *fp, SEXP_t *sexp, uint32_t indent)
         default:
                 abort ();
         }
-        
+
         /* NOTREACHED */
         return (-1);
 }
@@ -903,4 +904,3 @@ int SEXP_asnprintft (char **ret, size_t n, SEXP_t *sexp)
         return (-1);
 }
 #endif
-#endif
