@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -56,79 +56,94 @@
 #include <errno.h>
 
 #if defined(__linux__)
-#include <arpa/inet.h>
-#include <netlink/route/link.h>
-#include <netlink/route/addr.h>
+#include <sys/socket.h>
+#include <ifaddrs.h>
+#include <netdb.h>
+#include <sys/ioctl.h>
+#include <string.h>
+#include <net/if.h>
 
-/* Use netlink libnl 1.0 API */
-#ifdef HAVE_LIBNL10
-#define nl_cache_free nl_cache_destroy_and_free
-#endif
+static int fd=-1;
 
-struct nl_cache *link_cache;
-void get_ifs(SEXP_t *item);
-
-static void cb(struct nl_object *obj, void *arg)
+static char *get_mac(const struct ifaddrs *ifa)
 {
-        struct rtnl_addr *rtaddr = (struct rtnl_addr *) obj;
-        struct nl_addr *absaddr;
-        struct rtnl_link *rtlink;
-        int ifindex;
-        char iabuf[64], mabuf[64];
-        char *name = NULL;
-        SEXP_t *item = (SEXP_t *) arg;
-        SEXP_t *attrs, *r0, *r1, *r2;
+       struct ifreq ifr;
+       unsigned char mac[6];
+       static char mac_buf[20];
 
-        ifindex = rtnl_addr_get_ifindex(rtaddr);
-        name = rtnl_addr_get_label(rtaddr);
-        absaddr = rtnl_addr_get_local(rtaddr);
-        nl_addr2str(absaddr, iabuf, sizeof (iabuf));
+       memset(&ifr, 0, sizeof(struct ifreq));
+       strcpy(ifr.ifr_name, ifa->ifa_name);
+       if (ioctl(fd, SIOCGIFHWADDR, &ifr) >= 0) {
+               memcpy(mac, ifr.ifr_hwaddr.sa_data, sizeof(mac));
+               snprintf(mac_buf, sizeof(mac_buf),
+                       "%02X:%02X:%02X:%02X:%02X:%02X",
+                       mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+       } else
+               mac_buf[0] = 0;
 
-        rtlink = rtnl_link_get(link_cache, ifindex);
-        if (name == NULL)
-                name = rtnl_link_get_name(rtlink);
+       return mac_buf;
+}
+
+static int get_ifs(SEXP_t *item)
+{
+       struct ifaddrs *ifaddr, *ifa;
+       int family, rc=1;
+       char host[NI_MAXHOST], *mac;
+       SEXP_t *attrs;
+       SEXP_t *r0, *r1, *r2;
+
+       if (getifaddrs(&ifaddr) == -1)
+               return rc;
+
+       fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
+       if (fd < 0)
+               goto leave1;
+
+        /* Walk through linked list, maintaining head pointer so we
+          can free list later */
+        for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+                if (ifa->ifa_addr) {
+                        family = ifa->ifa_addr->sa_family;
+                        if (family != AF_INET && family != AF_INET6)
+                                continue;
+                } else
+                        continue;
+
+                mac = get_mac(ifa);
+                rc = getnameinfo(ifa->ifa_addr, (family == AF_INET) ?
+                        sizeof(struct sockaddr_in) :
+                        sizeof(struct sockaddr_in6), host, NI_MAXHOST,
+                        NULL, 0, NI_NUMERICHOST);
+                if (rc) {
+                        rc = 1;
+                        goto leave2;
+                }
 
-        absaddr = rtnl_link_get_addr(rtlink);
-        rtnl_link_put(rtlink);
-	if (absaddr && !nl_addr_iszero(absaddr)) {
-	        nl_addr2str(absaddr, mabuf, sizeof (mabuf));
-        
 	        attrs = probe_attr_creat("name",
-                                 r0 = SEXP_string_newf("%s", name),
+                                 r0 = SEXP_string_newf("%s", ifa->ifa_name),
                                  "ip_address",
-                                 r1 = SEXP_string_newf("%s", iabuf),
+                                 r1 = SEXP_string_newf("%s", host),
                                  "mac_address",
-                                 r2 = SEXP_string_newf("%s", mabuf),
+                                 r2 = SEXP_string_newf("%s", mac),
                                  NULL);
 	        probe_item_ent_add(item, "interface", attrs, NULL);
         	SEXP_vfree(attrs, r0, r1, r2, NULL);
 	}
+leave2:
+        close(fd);
+leave1:
+        freeifaddrs(ifaddr);
+        return rc;
 }
 
-void get_ifs(SEXP_t *item)
-{
-        struct nl_handle *sock;
-        struct nl_cache *addr_cache;
-
-        sock = nl_handle_alloc();
-        nl_connect(sock, NETLINK_ROUTE);
-        link_cache = rtnl_link_alloc_cache(sock);
-        addr_cache = rtnl_addr_alloc_cache(sock);
-
-        nl_cache_foreach(addr_cache, cb, (void *) item);
-
-        nl_cache_free(link_cache);
-        nl_cache_free(addr_cache);
-        nl_close(sock);
-        nl_handle_destroy(sock);
-}
 #else
-void get_ifs(SEXP_t *item)
+
+int get_ifs(SEXP_t *item)
 {
         /* TODO */
 
         SEXP_t *attrs, *r0, *r1, *r2;
-        
+
         attrs = probe_attr_creat("name",
                                  r0 = SEXP_string_newf("dummy0"),
                                  "ip_address",
@@ -136,11 +151,11 @@ void get_ifs(SEXP_t *item)
                                  "mac_address",
                                  r2 = SEXP_string_newf("aa:bb:cc:dd:ee:ff"),
                                  NULL);
-        
+
         probe_item_ent_add(item, "interface", attrs, NULL);
         SEXP_vfree (attrs, r0, r1, r2, NULL);
-        
-        return;
+
+        return 0;
 }
 #endif
 
@@ -150,7 +165,7 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
         SEXP_t *r0, *r1, *r2, *r3;
         char *os_name, *os_version, *architecture, *hname;
         struct utsname sname;
-        
+
         (void)arg;
 
 	if (probe_in == NULL) {
@@ -176,7 +191,10 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
                                   "primary_host_name", NULL, r3 = SEXP_string_newf("%s", hname),
                                   NULL);
 
-        get_ifs(item);
+        if (get_ifs(item)) {
+               *err = PROBE_EUNKNOWN;
+               return NULL;
+       }
         SEXP_vfree (r0, r1, r2, r3, NULL);
 	*err = 0;
 
