@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +21,6 @@
  */
 
 
-#ifndef __STUB_PROBE
 /*
  * textfilecontent54 probe:
  *
@@ -62,6 +61,7 @@
 #include <probe-entcmp.h>
 #include <findfile.h>
 #include <alloc.h>
+#include "common/assume.h"
 
 #define FILE_SEPARATOR '/'
 
@@ -184,27 +184,13 @@ static SEXP_t *create_item(const char *path, const char *filename, char *pattern
         SEXP_vfree (r0, r1, r2, r3, r4, NULL);
 
 	for (i = 1; i < substr_cnt; ++i) {
-                probe_item_ent_add (item, "subexpression", NULL, r0 = SEXP_string_newf(substrs[i]));
+                probe_item_ent_add (item, "subexpression", NULL, r0 = SEXP_string_new (substrs[i], strlen (substrs[i])));
                 SEXP_free (r0);
 	}
 
 	return item;
 }
 
-static int report_missing(SEXP_t *ent)
-{
-	oval_operation_t op;
-        SEXP_t *r0;
-
-	op = SEXP_number_geti_32 (r0 = probe_ent_getattrval(ent, "operation"));
-        SEXP_free (r0);
-
-        if (op == OVAL_OPERATION_EQUALS)
-		return 1;
-	else
-		return 0;
-}
-
 struct pfdata {
 	char *pattern;
 	SEXP_t *filename_ent;
@@ -212,12 +198,15 @@ struct pfdata {
 	SEXP_t *item_list;
 };
 
-static int process_file(const char *path, const char *filename, void *arg)
+static int process_file(const char *path, const char *file, void *arg)
 {
 	struct pfdata *pfd = (struct pfdata *) arg;
-	int ret = 0, path_len, filename_len;
+	int ret = 0, path_len, file_len;
 	char *whole_path = NULL;
 	FILE *fp = NULL;
+	int  cur_inst = 0;
+	char line[4096];
+	SEXP_t *next_inst = NULL;
 
 // todo: move to probe_main()?
 #if defined USE_REGEX_PCRE
@@ -236,43 +225,21 @@ static int process_file(const char *path, const char *filename, void *arg)
 		return -1;
 	}
 #endif
-
-	if (filename == NULL) {
-		/*
-		SEXP_t *item;
-                SEXP_t *r0, *r1;
-
-		if (report_missing(pfd->filename_ent)) {
-			item = probe_item_creat ("textfilecontent_item", NULL,
-                                                 "path", NULL, r0 = SEXP_string_newf("%s", path),
-                                                 "filename", NULL, r1 = probe_ent_getval(pfd->filename_ent),
-                                                 NULL);
-                        SEXP_vfree (r0, r1, NULL);
-                        probe_item_setstatus (item, OVAL_STATUS_DOESNOTEXIST);
-		        probe_itement_setstatus (item, "filename", 1, OVAL_STATUS_DOESNOTEXIST);
-		} else {
-			item = probe_item_creat ("textfilecontent_item", NULL,
-                                                 "path", NULL, r0 = SEXP_string_newf("%s", path),
-                                                 NULL);
-                        SEXP_free(r0);
-		}
-
-		SEXP_list_add(pfd->item_list, item);
-                SEXP_free(item);
-		*/
-
+	if (file == NULL)
 		goto cleanup;
-	}
 
-	path_len = strlen(path);
-	filename_len = strlen(filename);
-	whole_path = oscap_alloc(path_len + filename_len + 2);
+	path_len   = strlen(path);
+	file_len   = strlen(file);
+	whole_path = oscap_alloc(path_len + file_len + 2);
+
 	memcpy(whole_path, path, path_len);
+
 	if (whole_path[path_len - 1] != FILE_SEPARATOR) {
 		whole_path[path_len] = FILE_SEPARATOR;
 		++path_len;
 	}
-	memcpy(whole_path + path_len, filename, filename_len + 1);
+
+	memcpy(whole_path + path_len, file, file_len + 1);
 
 	fp = fopen(whole_path, "rb");
 	if (fp == NULL) {
@@ -280,23 +247,20 @@ static int process_file(const char *path, const char *filename, void *arg)
 		goto cleanup;
 	}
 
-	int cur_inst = 0;
-	char line[4096];
-	SEXP_t *next_inst = NULL;
-
 	while (fgets(line, sizeof(line), fp) != NULL) {
 		char **substrs;
 		int substr_cnt, want_instance;
 
-		if (next_inst != NULL)
-			SEXP_free(next_inst);
 		next_inst = SEXP_number_newi_32(cur_inst + 1);
+
 		if (probe_entobj_cmp(pfd->instance_ent, next_inst) == OVAL_RESULT_TRUE)
 			want_instance = 1;
 		else
 			want_instance = 0;
 
+                SEXP_free(next_inst);
 		substr_cnt = get_substrings(line, re, want_instance, &substrs);
+
 		if (substr_cnt > 0) {
 			++cur_inst;
 
@@ -304,10 +268,11 @@ static int process_file(const char *path, const char *filename, void *arg)
 				int k;
                                 SEXP_t *item;
 
-                                item = create_item(path, filename, pfd->pattern,
+                                item = create_item(path, file, pfd->pattern,
                                                    cur_inst, substrs, substr_cnt);
                                 SEXP_list_add(pfd->item_list, item);
                                 SEXP_free(item);
+
 				for (k = 0; k < substr_cnt; ++k)
 					free(substrs[k]);
 				free(substrs);
@@ -326,15 +291,16 @@ static int process_file(const char *path, const char *filename, void *arg)
 #elif defined USE_REGEX_POSIX
 	regfree(re);
 #endif
-
 	return ret;
 }
 
 SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 {
-	SEXP_t *path_ent, *filename_ent, *instance_ent, *behaviors_ent;
+	SEXP_t *path_ent, *file_ent, *inst_ent, *bh_ent, *patt_ent;
         SEXP_t *r0, *r1;
 	char *pattern;
+	int fcnt;
+	struct pfdata pfd;
 
         (void)arg;
 
@@ -343,69 +309,66 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 		return NULL;
 	}
 
-	/* parse request */
-        r0 = NULL;
-	if ( (path_ent = probe_obj_getent(probe_in, "path", 1)) == NULL ||
-	     (filename_ent = probe_obj_getent(probe_in, "filename", 1)) == NULL ||
-	     (pattern = SEXP_string_cstr(r0 = probe_obj_getentval(probe_in, "pattern", 1))) == NULL ||
-	     (instance_ent = probe_obj_getent(probe_in, "instance", 1)) == NULL) {
-                if (r0 != NULL)
-                        SEXP_free(r0);
-		*err = PROBE_ENOELM;
-		return NULL;
-	}
-        SEXP_free(r0);
+        *err = PROBE_ENOELM;
+
+        if ((path_ent = probe_obj_getent(probe_in, "path",     1)) == NULL)
+                return(NULL);
+        if ((file_ent = probe_obj_getent(probe_in, "filename", 1)) == NULL) {
+                SEXP_free(path_ent);
+                return(NULL);
+        }
+        if ((inst_ent = probe_obj_getent(probe_in, "instance", 1)) == NULL) {
+                SEXP_free(path_ent);
+                SEXP_free(file_ent);
+                return(NULL);
+        }
+        if ((patt_ent = probe_obj_getent(probe_in, "pattern",  1)) == NULL) {
+                SEXP_free(path_ent);
+                SEXP_free(file_ent);
+                SEXP_free(inst_ent);
+                return(NULL);
+        } else {
+                SEXP_t *ent_val;
+
+                ent_val = probe_ent_getval(patt_ent);
+                pattern = SEXP_string_cstr(ent_val);
+                assume_d(pattern != NULL, -1);
+                SEXP_free(patt_ent);
+                SEXP_free(ent_val);
+        }
 
 	/* canonicalize behaviors */
-	behaviors_ent = probe_obj_getent(probe_in, "behaviors", 1);
-	if (behaviors_ent == NULL) {
-		SEXP_t * behaviors_new;
-		behaviors_new = probe_ent_creat("behaviors",
-                                                probe_attr_creat("max_depth", r0 = SEXP_string_newf ("1"),
-                                                                 "recurse_direction", r1 = SEXP_string_newf ("none"),
-                                                                 NULL),
-                                                NULL /* val */,
-                                                NULL /* end */);
-		behaviors_ent = SEXP_list_first(behaviors_new);
-                SEXP_vfree(r0, r1, behaviors_new, NULL);
-	}
-	else {
-		if (!probe_ent_attrexists (behaviors_ent, "max_depth")) {
-			probe_ent_attr_add (behaviors_ent,"max_depth", r0 = SEXP_string_newf ("1"));
+	bh_ent = probe_obj_getent(probe_in, "behaviors", 1);
+
+	if (bh_ent == NULL) {
+		SEXP_t *bh_new, *bh_attr;
+
+                bh_attr = probe_attr_creat("max_depth",         r0 = SEXP_string_newf ("1"),
+                                           "recurse_direction", r1 = SEXP_string_newf ("none"),
+                                           NULL);
+		bh_new  = probe_ent_creat("behaviors", bh_attr, NULL /* val */, NULL /* end */);
+		bh_ent  = SEXP_list_first(bh_new);
+
+                SEXP_vfree(bh_new, bh_attr, r0, r1, NULL);
+	} else {
+		if (!probe_ent_attrexists (bh_ent, "max_depth")) {
+			probe_ent_attr_add (bh_ent,"max_depth", r0 = SEXP_string_newf ("1"));
                         SEXP_free (r0);
                 }
-		if (!probe_ent_attrexists (behaviors_ent, "recurse_direction")) {
-			probe_ent_attr_add (behaviors_ent,"recurse_direction", r0 = SEXP_string_newf ("none"));
+		if (!probe_ent_attrexists (bh_ent, "recurse_direction")) {
+			probe_ent_attr_add (bh_ent,"recurse_direction", r0 = SEXP_string_newf ("none"));
                         SEXP_free (r0);
                 }
 	}
 
-	int fcnt;
-	struct pfdata pfd;
+	pfd.pattern      = pattern;
+	pfd.filename_ent = file_ent;
+	pfd.instance_ent = inst_ent;
+	pfd.item_list    = SEXP_list_new(NULL);
 
-	pfd.pattern = pattern;
-	pfd.filename_ent = filename_ent;
-	pfd.instance_ent = instance_ent;
-	pfd.item_list = SEXP_list_new(NULL);
+	fcnt = find_files(path_ent, file_ent, bh_ent, process_file, (void *) &pfd);
 
-	fcnt = find_files(path_ent, filename_ent, behaviors_ent,
-			  process_file, (void *) &pfd);
-	if (fcnt == 0) {
-		/*
-		if (report_missing(pfd.filename_ent)) {
-			SEXP_t *item;
-
-			item = probe_item_creat("textfilecontent_item", NULL,
-                                                "path", NULL,
-                                                r0 = probe_ent_getval(path_ent),
-                                                NULL);
-                        probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
-			probe_itement_setstatus(item, "path", 1, OVAL_STATUS_DOESNOTEXIST);
-			SEXP_list_add(pfd.item_list, item);
-                        SEXP_vfree(r0, item, NULL);
-		}
-		*/
-	} else if (fcnt < 0) {
+	if (fcnt < 0) {
 		SEXP_t *item;
 		item = probe_item_creat("textfilecontent_item", NULL,
                                         "path", NULL,
@@ -416,7 +379,13 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
                 SEXP_vfree(r0, item, NULL);
 	}
 
+        SEXP_free(file_ent);
+        SEXP_free(path_ent);
+        SEXP_free(inst_ent);
+        SEXP_free(bh_ent);
+        oscap_free(pattern);
+
 	*err = 0;
 	return pfd.item_list;
 }
-#endif
+
