@@ -1,6 +1,5 @@
-
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +20,6 @@
  *      "Tomas Heinrich" <theinric@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 /*
  * xmlfilecontent probe:
  *
@@ -86,11 +84,11 @@ static int report_missing(SEXP_t *ent)
 {
 	oval_operation_t op;
         SEXP_t *r0;
-        
+
         r0 = probe_ent_getattrval(ent, "operation");
 	op = SEXP_number_geti_32(r0);
         SEXP_free (r0);
-                
+
 	if (op == OVAL_OPERATION_EQUALS)
 		return 1;
 	else
@@ -109,51 +107,51 @@ static int process_file(const char *path, const char *filename, void *arg)
 	xmlNodePtr cur_node, *node_tab;
 	xmlChar *value;
 	SEXP_t *item;
-        
+
         SEXP_t *r0, *r1, *r2;
 
 	if (filename == NULL) {
 		if (report_missing(pfd->filename_ent)) {
                         item = probe_item_creat("xmlfilecontent_item", NULL,
                                                 /* entities */
-                                                "path",     NULL, r0 = SEXP_string_newf(path),
-                                                "filename", NULL, r1 = SEXP_string_newf(filename),
+                                                "path",     NULL, r0 = SEXP_string_new (path, strlen (path)),
+                                                "filename", NULL, r1 = SEXP_string_new (filename, strlen (filename)),
                                                 NULL);
-                        
+
                         SEXP_vfree (r0, r1, NULL);
-                        
+
                         probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
                         probe_itement_setstatus(item, "filename", 1, OVAL_STATUS_DOESNOTEXIST);
 		} else {
                         item = probe_item_creat("xmlfilecontent_item", NULL,
                                                 /* entities */
-                                                "path", NULL, r0 = SEXP_string_newf(path),
+                                                "path", NULL, r0 = SEXP_string_new (path, strlen (path)),
                                                 NULL);
-                        
+
                         SEXP_free (r0);
 		}
-                
+
 		SEXP_list_add(pfd->item_list, item);
                 SEXP_free (item);
-                
+
 		goto cleanup;
 	}
-        
+
 	path_len     = strlen(path);
 	filename_len = strlen(filename);
 	whole_path   = malloc(sizeof (char) * (path_len + filename_len + 2));
-        
+
 	memcpy (whole_path, path, sizeof (char) * path_len);
-        
+
 	if (whole_path[path_len - 1] != FILE_SEPARATOR) {
 		whole_path[path_len] = FILE_SEPARATOR;
 		++path_len;
 	}
-        
+
 	memcpy(whole_path + path_len, filename, filename_len + 1);
 
 	/* evaluate xpath */
-        
+
 	doc = xmlParseFile(whole_path);
 	if (doc == NULL) {
 		ret = -1;
@@ -180,9 +178,9 @@ static int process_file(const char *path, const char *filename, void *arg)
 
         item = probe_item_creat("xmlfilecontent_item", NULL,
                                 /* entities */
-                                "path",     NULL, r0 = SEXP_string_newf(path),
-                                "filename", NULL, r1 = SEXP_string_newf(filename),
-                                "xpath",    NULL, r2 = SEXP_string_newf(pfd->xpath),
+                                "path",     NULL, r0 = SEXP_string_new(path, strlen (path)),
+                                "filename", NULL, r1 = SEXP_string_new(filename, strlen (filename)),
+                                "xpath",    NULL, r2 = SEXP_string_new(pfd->xpath, strlen (pfd->xpath)),
                                 NULL);
 
         SEXP_vfree (r0, r1, r2, NULL);
@@ -200,16 +198,16 @@ static int process_file(const char *path, const char *filename, void *arg)
 			    cur_node->type == XML_TEXT_NODE)
                         {
 				value = xmlNodeGetContent(cur_node);
-				probe_item_ent_add(item, "value_of", NULL, r0 = SEXP_string_newf((char *) value));
+				probe_item_ent_add(item, "value_of", NULL, r0 = SEXP_string_new ((char *) value, strlen ((char *) value)));
 				xmlFree(value);
                                 SEXP_free (r0);
 			}
 		}
 	}
-        
+
 	SEXP_list_add(pfd->item_list, item);
         SEXP_free (item);
-        
+
  cleanup:
 	if (xpath_obj != NULL)
 		xmlXPathFreeObject(xpath_obj);
@@ -219,7 +217,7 @@ static int process_file(const char *path, const char *filename, void *arg)
 		xmlFreeDoc(doc);
 	if (whole_path != NULL)
 		free(whole_path);
-        
+
 	return ret;
 }
 
@@ -227,7 +225,7 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 {
 	SEXP_t *path_ent, *filename_ent, *xpath_ent, *behaviors_ent;
         SEXP_t *r0, *r1, *r2;
-        
+
         (void)arg;
 
 	if (probe_in == NULL) {
@@ -243,19 +241,19 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 		*err = PROBE_ENOELM;
 		return NULL;
 	}
-        
+
 	/* canonicalize behaviors */
 	behaviors_ent = probe_obj_getent(probe_in, "behaviors", 1);
         if (behaviors_ent == NULL) {
 		SEXP_t *behaviors_new;
-                
+
 		behaviors_new = probe_ent_creat("behaviors",
                                                 r0 = probe_attr_creat("max_depth", r1 = SEXP_string_newf ("1"),
                                                                       "recurse_direction", r2 = SEXP_string_newf ("none"),
                                                                       NULL),
                                                 NULL /* val */,
                                                 NULL /* end */);
-                
+
                 SEXP_vfree (r0, r1, r2, NULL);
 
 		behaviors_ent = SEXP_list_first(behaviors_new);
@@ -265,22 +263,22 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 			probe_ent_attr_add (behaviors_ent,"max_depth", r0 = SEXP_string_newf ("1"));
                         SEXP_free (r0);
                 }
-                
+
 		if (!probe_ent_attrexists (behaviors_ent, "recurse_direction")) {
 			probe_ent_attr_add (behaviors_ent,"recurse_direction", r0 = SEXP_string_newf ("none"));
                         SEXP_free (r0);
                 }
 	}
-        
+
 	int fcnt;
 	struct pfdata pfd;
-        
+
 	pfd.xpath = SEXP_string_cstr(r0 = probe_ent_getval(xpath_ent));
         SEXP_free (r0);
-        
+
 	pfd.filename_ent = filename_ent;
 	pfd.item_list = SEXP_list_new(NULL);
-        
+
 	fcnt = find_files(path_ent, filename_ent, behaviors_ent,
 			  process_file, (void *) &pfd);
 	if (fcnt == 0) {
@@ -292,7 +290,7 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
                                                 "path", NULL, r0 = probe_ent_getval(path_ent),
                                                 NULL);
                         SEXP_free (r0);
-                        
+
                         probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
 			probe_itement_setstatus(item, "path", 1, OVAL_STATUS_DOESNOTEXIST);
 			SEXP_list_add(pfd.item_list, item);
@@ -300,7 +298,7 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 		}
 	} else if (fcnt < 0) {
 		SEXP_t *item;
-                
+
 		item = probe_item_creat("xmlfilecontent_item", NULL,
                                         /* entities */
                                         "path", NULL, r0 = probe_ent_getval(path_ent),
@@ -310,13 +308,12 @@ SEXP_t *probe_main(SEXP_t *probe_in, int *err, void *arg)
 		SEXP_list_add(pfd.item_list, item);
                 SEXP_free (item);
 	}
-        
+
         SEXP_free (path_ent);
         SEXP_free (filename_ent);
         SEXP_free (xpath_ent);
         SEXP_free (behaviors_ent);
-        
+
 	*err = 0;
 	return pfd.item_list;
 }
-#endif
