@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,7 +25,6 @@
  *      Daniel Kopecek <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
@@ -84,20 +83,70 @@ SEXP_t *probe_item_build(const char *fmt, ...)
 	return (NULL);
 }
 
-/*
-  SEXP_t *probe_item_creat (const char *name, SEXP_t *attrs, ...)
-  {
-  return probe_obj_creat (name, attrs, ...);
-  }
-*/
+SEXP_t *probe_item_creat(const char *name, SEXP_t * attrs, ...)
+{
+	va_list ap;
+	SEXP_t *itm, *ns, *val, *ent;
+
+	_LOGCALL_;
+
+	va_start(ap, attrs);
+
+	itm = probe_item_new(name, attrs);
+	name = va_arg(ap, const char *);
+
+	while (name != NULL) {
+		attrs = va_arg(ap, SEXP_t *);
+		val = va_arg(ap, SEXP_t *);
+
+                ns  = encache_ref (OSCAP_GSYM(encache), name);
+		ent = SEXP_list_new(NULL);
+
+		if (attrs != NULL) {
+			SEXP_t *nl, *nj;
+
+			nl = SEXP_list_new(ns, NULL);
+			nj = SEXP_list_join(nl, attrs);
+
+			SEXP_list_add(ent, nj);
+			SEXP_free(nl);
+			SEXP_free(nj);
+		} else
+			SEXP_list_add(ent, ns);
+
+		SEXP_free(ns);
+
+		SEXP_list_add(ent, val);
+		SEXP_list_add(itm, ent);
+
+		SEXP_free(ent);
+
+		name = va_arg(ap, const char *);
+	}
+
+	return (itm);
+}
 
 SEXP_t *probe_item_new(const char *name, SEXP_t * attrs)
 {
+	SEXP_t *itm, *sid, *attr;
+
 	_LOGCALL_;
+
+	sid = probe_item_newid(&(OSCAP_GSYM(id_desc)));
+	attr = probe_attr_creat("id", sid, NULL);
+	if (attrs != NULL) {
+		attrs = SEXP_list_join(attr, attrs);
+		SEXP_free(attr);
+	} else
+		attrs = attr;
 	/*
 	 * Objects have the same structure as items.
 	 */
-	return probe_obj_new(name, attrs);
+	itm = probe_obj_new(name, attrs);
+	SEXP_vfree(sid, attrs, NULL);
+
+	return itm;
 }
 
 SEXP_t *probe_item_attr_add(SEXP_t * item, const char *name, SEXP_t * val)
@@ -206,7 +255,7 @@ SEXP_t *probe_item_newid(struct id_desc_t * id_desc)
 	return sid;
 }
 
-static void probe_item_resetidctr(struct id_desc_t *id_desc)
+void probe_item_resetidctr(struct id_desc_t *id_desc)
 {
 	id_desc->item_id_ctr = 1;
 }
@@ -275,7 +324,7 @@ SEXP_t *probe_obj_creat(const char *name, SEXP_t * attrs, ...)
 
                 ns  = encache_ref (OSCAP_GSYM(encache), name);
 		ent = SEXP_list_new(NULL);
-                
+
 		if (attrs != NULL) {
 			SEXP_t *nl, *nj;
 
@@ -311,13 +360,14 @@ SEXP_t *probe_obj_new(const char *name, SEXP_t * attrs)
 	ns  = encache_ref (OSCAP_GSYM(encache), name);
 
 	if (attrs != NULL) {
-		SEXP_t *nl;
+		SEXP_t *nl, *nj;
 
 		nl = SEXP_list_new(ns, NULL);
-		nl = SEXP_list_join(nl, attrs);
+		nj = SEXP_list_join(nl, attrs);
 
-		SEXP_list_add(obj, nl);
+		SEXP_list_add(obj, nj);
 		SEXP_free(nl);
+                SEXP_free(nj);
 	} else
 		SEXP_list_add(obj, ns);
 
@@ -808,7 +858,10 @@ SEXP_t *probe_ent_attr_add(SEXP_t * ent, const char *name, SEXP_t * val)
 int probe_ent_getvals(const SEXP_t * ent, SEXP_t ** res)
 {
 	_LOGCALL_;
-	(*res) = SEXP_list_rest(ent);
+	if (probe_ent_attrexists(ent, "var_ref"))
+		(*res) = SEXP_list_nth(ent, 2);
+	else
+		(*res) = SEXP_list_rest(ent);
 	return (SEXP_list_length(*res));
 }
 
@@ -863,7 +916,9 @@ SEXP_t *probe_ent_getattrval(const SEXP_t * ent, const char *name)
 					if (attr_name[0] == ':') {
 						if (strcmp(attr_name + 1, name) == 0) {
 							SEXP_free(attr);
-							return SEXP_list_nth(attrs, i + 1);
+							attr = SEXP_list_nth(attrs, i + 1);
+							SEXP_free(attrs);
+							return attr;
 						}
 					}
 				}
@@ -875,6 +930,7 @@ SEXP_t *probe_ent_getattrval(const SEXP_t * ent, const char *name)
 		}
 	}
 
+	SEXP_free(attrs);
 	return (NULL);
 }
 
@@ -1132,4 +1188,4 @@ void probe_free(SEXP_t * obj)
 {
 	SEXP_free(obj);
 }
-#endif
+
