@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,10 +21,10 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#include <signal.h>
 #include <errno.h>
 #include <config.h>
 #include <alloc.h>
@@ -36,11 +36,15 @@
 #define _A(x) assert(x)
 #endif
 
-SEAP_CTX_t *OSCAP_GSYM(ctx)       = NULL;
-int         OSCAP_GSYM(sd)        = -1;
-pcache_t   *OSCAP_GSYM(pcache)    = NULL;
-void       *OSCAP_GSYM(probe_arg) = NULL;
-encache_t  *OSCAP_GSYM(encache)   = NULL;
+#define PROBE_SIGEXIT_CLEAN   1
+#define PROBE_SIGEXIT_UNCLEAN 2
+
+volatile int OSCAP_GSYM(sigexit)   = 0;
+SEAP_CTX_t  *OSCAP_GSYM(ctx)       = NULL;
+int          OSCAP_GSYM(sd)        = -1;
+pcache_t    *OSCAP_GSYM(pcache)    = NULL;
+void        *OSCAP_GSYM(probe_arg) = NULL;
+encache_t   *OSCAP_GSYM(encache)   = NULL;
 
 struct id_desc_t OSCAP_GSYM(id_desc);
 
@@ -529,6 +533,31 @@ static SEXP_t *probe_set_eval(SEXP_t * set, size_t depth)
 	return (NULL);
 }
 
+static void probe_cleanup(void)
+{
+	probe_fini(OSCAP_GSYM(probe_arg));
+	pcache_free(OSCAP_GSYM(pcache));
+        encache_free(OSCAP_GSYM(encache));
+	SEAP_close(OSCAP_GSYM(ctx), OSCAP_GSYM(sd));
+	SEAP_CTX_free(OSCAP_GSYM(ctx));
+}
+
+static void probe_sigexit(int signum)
+{
+        switch(signum) {
+        case SIGTERM:
+        case SIGINT:
+        case SIGPIPE:
+        case SIGQUIT:
+                OSCAP_GSYM(sigexit) = PROBE_SIGEXIT_CLEAN;
+                break;
+        case SIGSEGV:
+        case SIGABRT:
+                OSCAP_GSYM(sigexit) = PROBE_SIGEXIT_UNCLEAN;
+                break;
+        }
+}
+
 int main(void)
 {
 	int ret = EXIT_SUCCESS;
@@ -542,6 +571,31 @@ int main(void)
 	pthread_attr_t thread_attr;
 	pthread_t thread;
 
+        struct sigaction sigact;
+
+        /* Setup signal handler */
+        sigemptyset(&sigact.sa_mask);
+        sigaddset(&sigact.sa_mask, SIGTERM);
+        sigaddset(&sigact.sa_mask, SIGINT);
+        sigaddset(&sigact.sa_mask, SIGPIPE);
+        sigaddset(&sigact.sa_mask, SIGQUIT);
+        sigaddset(&sigact.sa_mask, SIGSEGV);
+        sigaddset(&sigact.sa_mask, SIGABRT);
+
+        sigact.sa_handler = &probe_sigexit;
+        sigact.sa_flags   = 0;
+
+        if (sigaction(SIGTERM, &sigact, NULL) != 0 ||
+            sigaction(SIGINT,  &sigact, NULL) != 0 ||
+            sigaction(SIGPIPE, &sigact, NULL) != 0 ||
+            sigaction(SIGQUIT, &sigact, NULL) != 0 ||
+            sigaction(SIGSEGV, &sigact, NULL) != 0 ||
+            sigaction(SIGABRT, &sigact, NULL) != 0)
+        {
+                _D("Can't setup signal handlers: errno=%u, %s.\n", errno, strerror(errno));
+                exit(errno);
+        }
+
 	/* Initialize SEAP */
 	OSCAP_GSYM(ctx) = SEAP_CTX_new();
 	OSCAP_GSYM(sd) = SEAP_openfd2(OSCAP_GSYM(ctx), STDIN_FILENO, STDOUT_FILENO, 0);
@@ -569,25 +623,33 @@ int main(void)
         }
 
 	OSCAP_GSYM(probe_arg) = probe_init();
-        
+
         /* Create the element name cache */
         OSCAP_GSYM(encache) = encache_new ();
 
+	/* Reset the item id generator */
+	probe_item_resetidctr(&(OSCAP_GSYM(id_desc)));
+
+        /* Register cleanup function */
+        atexit(&probe_cleanup);
+
 	/* Main loop */
-	for (;;) {
+	while(OSCAP_GSYM(sigexit) == 0) {
 		if (SEAP_recvmsg(OSCAP_GSYM(ctx), OSCAP_GSYM(sd), &seap_request) == -1) {
 			ret = errno;
-
 			_D("An error ocured while receiving SEAP message. errno=%u, %s.\n", errno, strerror(errno));
+                        switch(OSCAP_GSYM(sigexit)) {
+                        case PROBE_SIGEXIT_CLEAN:
+                                ret = 0;
+                                break;
+                        case PROBE_SIGEXIT_UNCLEAN:
+                                /* Try to save cached data & restart? */
+                                _exit(ret);
+                        }
 
 			break;
 		}
 
-/*
-#ifndef NDEBUG
-                SEAP_msg_print (stderr, seap_request);
-#endif  
-*/
 		probe_in = SEAP_msg_get(seap_request);
 		if (probe_in == NULL) {
 			_D("Unexpected error: probe_in = NULL\n");
@@ -595,8 +657,8 @@ int main(void)
 		}
 
 		SEXP_VALIDATE(probe_in);
-
 		oid = probe_obj_getattrval(probe_in, "id");
+                SEXP_free(probe_in);
 
 		if (oid == NULL) {
 			_D("Invalid object: %s\n", "attribute \"id\" not set\n");
@@ -605,6 +667,8 @@ int main(void)
 			SEXP_VALIDATE(oid);
 
 			probe_out = pcache_sexp_get(OSCAP_GSYM(pcache), oid);
+                        SEXP_free(oid);
+
 			if (probe_out == NULL) {
 				/* cache miss */
 
@@ -616,9 +680,6 @@ int main(void)
 				}
 
 				seap_request = NULL;
-
-				_D("New worker: id=%u, in=%p\n", thread, probe_in);
-
 				continue;
 			} else {
 				/* cache hit */
@@ -632,6 +693,10 @@ int main(void)
 				   errno, strerror(errno));
 
 				SEAP_msg_free(seap_request);
+
+                                if (probe_out != NULL)
+                                        SEXP_free(probe_out);
+
 				break;
 			}
 		} else {
@@ -639,6 +704,7 @@ int main(void)
 
 			seap_reply = SEAP_msg_new();
 			SEAP_msg_set(seap_reply, probe_out);
+                        SEXP_free(probe_out);
 
 			if (SEAP_reply(OSCAP_GSYM(ctx), OSCAP_GSYM(sd), seap_reply, seap_request) == -1) {
 				ret = errno;
@@ -646,8 +712,19 @@ int main(void)
 				_D("An error ocured while sending SEAP message. errno=%u, %s.\n",
 				   errno, strerror(errno));
 
-				SEAP_msg_free(seap_reply);
-				SEAP_msg_free(seap_request);
+                                switch(OSCAP_GSYM(sigexit)) {
+                                case PROBE_SIGEXIT_CLEAN:
+                                        SEAP_msg_free(seap_reply);
+                                        SEAP_msg_free(seap_request);
+                                        ret = 0;
+                                        break;
+                                case PROBE_SIGEXIT_UNCLEAN:
+                                        /* Try to save cache & restart? */
+                                        _exit(ret);
+                                default:
+                                        SEAP_msg_free(seap_reply);
+                                        SEAP_msg_free(seap_request);
+                                }
 
 				break;
 			}
@@ -658,11 +735,6 @@ int main(void)
 		SEAP_msg_free(seap_request);
 	}
 
-	probe_fini(OSCAP_GSYM(probe_arg));
-	pcache_free(OSCAP_GSYM(pcache));
-	SEAP_close(OSCAP_GSYM(ctx), OSCAP_GSYM(sd));
-	SEAP_CTX_free(OSCAP_GSYM(ctx));
-
 	return (ret);
 }
 
@@ -690,10 +762,10 @@ static int probe_varref_create_ctx(const SEXP_t * probe_in, SEXP_t * varrefs, st
 	ent_cnt = SEXP_number_getu_32(r1 = SEXP_list_nth(varrefs, 3));
 	SEXP_vfree(r0, r1, NULL);
 
-	ctx = malloc(sizeof(struct probe_varref_ctx));
-	ctx->pi2 = SEXP_ref(probe_in);
+	ctx = oscap_talloc (struct probe_varref_ctx);
+	ctx->pi2 = SEXP_softref((SEXP_t *)probe_in);
 	ctx->ent_cnt = ent_cnt;
-	ctx->ent_lst = malloc(ent_cnt * sizeof(struct probe_varref_ctx_ent));
+	ctx->ent_lst = oscap_alloc(ent_cnt * sizeof (ctx->ent_lst[0]));
 
 	vidx_name = SEXP_string_new(":val_idx", 8);
 	vidx_val = SEXP_number_newu(0);
@@ -787,16 +859,15 @@ static int probe_varref_iterate_ctx(struct probe_varref_ctx *ctx)
 	r0 = SEXP_number_newu(0);
 
 	while (++(*next_val_idx) >= val_cnt) {
-		*next_val_idx = 0;
-		r1 = SEXP_list_replace(ent_name_sref, 3, r0);
-		SEXP_free(r1);
-
-		if (ent == ent_end) {
+		if (++ent == ent_end) {
 			SEXP_free(r0);
 			return 0;
 		}
 
-		++ent;
+		*next_val_idx = 0;
+		r1 = SEXP_list_replace(ent_name_sref, 3, r0);
+		SEXP_free(r1);
+
 		val_cnt = ent->val_cnt;
 		next_val_idx = &ent->next_val_idx;
 		ent_name_sref = ent->ent_name_sref;
@@ -888,6 +959,10 @@ void *probe_worker(void *arg)
 			_D("An error ocured while sending error status. errno=%u, %s.\n", errno, strerror(errno));
 
 			SEAP_msg_free(seap_request);
+                        SEXP_free(probe_in);
+
+                        if (probe_out != NULL)
+                                SEXP_free(probe_out);
 
 			/* FIXME */
 			exit(ret);
@@ -899,6 +974,7 @@ void *probe_worker(void *arg)
 
 		oid = probe_obj_getattrval(probe_in, "id");
 		_A(oid != NULL);
+                SEXP_free(probe_in);
 
 		if (pcache_sexp_add(OSCAP_GSYM(pcache), oid, probe_out) != 0) {
 			/* TODO */
@@ -917,14 +993,15 @@ void *probe_worker(void *arg)
 
 			SEAP_msg_free(seap_reply);
 			SEAP_msg_free(seap_request);
+                        SEXP_free(probe_out);
 
 			exit(ret);
 		}
 
 		SEAP_msg_free(seap_reply);
+                SEXP_free(probe_out);
 	}
 
 	SEAP_msg_free(seap_request);
 	return (NULL);
 }
-#endif
