@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +21,6 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <seap.h>
 #include <probe-api.h>
 #include <stdlib.h>
@@ -89,13 +88,13 @@ static int report_missing(SEXP_t *ent)
         SEXP_t *r0;
 
         r0 = probe_ent_getattrval(ent, "operation");
-        
+
         if (r0 == NULL)
                 return (1);
-        
+
 	op = SEXP_number_geti_32 (r0);
         SEXP_free (r0);
-        
+
         if (op == OVAL_OPERATION_EQUALS)
 		return (1);
 	else
@@ -114,28 +113,28 @@ static int file_cb (const char *p, const char *f, void *ptr)
         struct cbargs *args = (struct cbargs *) ptr;
         struct stat st;
         const char *st_path;
-                
+
         res = args->items;
-        
+
         if (f == NULL) {
 		/*
                 if (report_missing (args->filename_ent)) {
                         SEXP_t *r0;
-                        
+
                         item = probe_obj_creat ("file_item", NULL,
                                                 "path",     NULL, r0 = SEXP_string_newf ("%s", p),
                                                 "filename", NULL, NULL,
                                                 NULL);
-                        
+
                         probe_item_setstatus(item, OVAL_STATUS_DOESNOTEXIST);
                         probe_itement_setstatus(item, "filename", 1, OVAL_STATUS_DOESNOTEXIST);
-                        
+
                         SEXP_list_add (res, item);
                         SEXP_free (item);
-                        
+
                         return (0);
                 }
-                
+
                 st_path = p;
 		*/
 		return (0);
@@ -144,31 +143,31 @@ static int file_cb (const char *p, const char *f, void *ptr)
                 snprintf (path_buffer, sizeof path_buffer, "%s/%s", p, f);
                 st_path = path_buffer;
         }
-        
+
         if (stat (st_path, &st) == -1) {
                 _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
                 return (-1);
         } else {
                 SEXP_t *r0, *r1, *r3, *r4;
                 SEXP_t *r5, *r6, *r7, *r8;
-                                                
-                item = probe_obj_creat ("file_item", NULL,
-                                        /* entities */                                        
+
+                item = probe_item_creat ("file_item", NULL,
+                                        /* entities */
                                         "path", NULL,
                                         r0 = SEXP_string_newf ("%s", p),
-                                        
+
                                         "filename", NULL,
                                         r1 = (f != NULL ? SEXP_string_newf ("%s", f) : NULL),
-                                        
+
                                         "type", NULL,
                                         strfiletype (st.st_mode),
-                                        
+
                                         "group_id", NULL,
                                         r3 = SEXP_string_newf ("%hu", st.st_gid),
-                                        
+
                                         "user_id", NULL,
                                         r4 = SEXP_string_newf ("%hu", st.st_uid),
-                                        
+
                                         "a_time", NULL,
                                         r5 = SEXP_string_newf ("%u",
 #if defined(OS_FREEBSD)
@@ -177,7 +176,7 @@ static int file_cb (const char *p, const char *f, void *ptr)
                                                                (unsigned int)st.st_atim.tv_sec
 #endif
                                                 ),
-                                        
+
                                         "c_time", NULL,
                                         r6 = SEXP_string_newf ("%u",
 #if defined(OS_FREEBSD)
@@ -186,7 +185,7 @@ static int file_cb (const char *p, const char *f, void *ptr)
                                                                (unsigned int)st.st_ctim.tv_sec
 #endif
                                                 ),
-                                        
+
                                         "m_time", NULL,
                                         r7 = SEXP_string_newf ("%u",
 #if defined(OS_FREEBSD)
@@ -195,7 +194,7 @@ static int file_cb (const char *p, const char *f, void *ptr)
                                                                (unsigned int)st.st_ctim.tv_sec
 #endif
                                                 ),
-                                        
+
                                         "size", NULL,
 #if defined(_FILE_OFFSET_BITS)
 # if   _FILE_OFFSET_BITS == 64
@@ -213,49 +212,49 @@ static int file_cb (const char *p, const char *f, void *ptr)
 #endif
                                         "suid", NULL,
                                         (st.st_mode & S_ISUID ? gr_true : gr_false),
-                                        
+
                                         "sticky", NULL,
                                         (st.st_mode & S_ISVTX ? gr_true : gr_false),
-                                        
+
                                         "uread", NULL,
                                         (st.st_mode & S_IRUSR ? gr_true : gr_false),
-                                        
+
                                         "uwrite", NULL,
                                         (st.st_mode & S_IWUSR ? gr_true : gr_false),
-                                        
+
                                         "uexec", NULL,
                                         (st.st_mode & S_IXUSR ? gr_true : gr_false),
-                                        
+
                                         "gread", NULL,
                                         (st.st_mode & S_IRGRP ? gr_true : gr_false),
-                                        
+
                                         "gwrite", NULL,
                                         (st.st_mode & S_IWGRP ? gr_true : gr_false),
-                                        
+
                                         "gexec", NULL,
                                         (st.st_mode & S_IXGRP ? gr_true : gr_false),
-                                        
+
                                         "oread", NULL,
                                         (st.st_mode & S_IROTH ? gr_true : gr_false),
-                                        
+
                                         "owrite", NULL,
                                         (st.st_mode & S_IWOTH ? gr_true : gr_false),
 
                                         "oexec", NULL,
                                         (st.st_mode & S_IXOTH ? gr_true : gr_false),
-                                        
+
                                         NULL);
-                
+
                 SEXP_list_add (res, item);
-                
+
                 _D("item memory size = %zu bytes\n", SEXP_sizeof (item));
                 _D("list memory size = %zu bytes\n", SEXP_sizeof (res));
-                
+
                 SEXP_vfree (item,
                             r0, r1, r3, r4,
                             r5, r6, r7, r8, NULL);
         }
-        
+
         return (0);
 }
 
@@ -264,13 +263,13 @@ static pthread_mutex_t __file_probe_mutex;
 void *probe_init (void)
 {
         _LOGCALL_;
-        
+
         /*
          * Initialize true/false global reference.
          */
         gr_true  = SEXP_number_newb (true);
         gr_false = SEXP_number_newb (false);
-        
+
         /*
          * Initialize file type string references.
          * (Used by strfiletype())
@@ -292,14 +291,14 @@ void *probe_init (void)
         default:
                 _D("Can't initialize mutex: errno=%u, %s.\n", errno, strerror (errno));
         }
-        
+
         return (NULL);
 }
 
 void probe_fini (void *arg)
 {
         _A((void *)arg == (void *)&__file_probe_mutex);
-        
+
         /*
          * Release global reference.
          */
@@ -307,12 +306,12 @@ void probe_fini (void *arg)
                     gr_t_dir, gr_t_lnk, gr_t_blk,
                     gr_t_fifo, gr_t_sock, gr_t_char,
                     NULL);
-        
+
         /*
          * Destroy mutex.
          */
         (void) pthread_mutex_destroy (&__file_probe_mutex);
-        
+
         return;
 }
 
@@ -322,31 +321,31 @@ SEXP_t *probe_main (SEXP_t *probe_in, int *err, void *mutex)
         SEXP_t *r0, *r1, *r2, *r3, *r4;
         int     filecnt;
         struct cbargs cbargs;
-        
+
         if (mutex == NULL) {
                 *err = PROBE_EINIT;
                 return (NULL);
         }
-        
+
         _A(mutex == &__file_probe_mutex);
-        
+
         path      = probe_obj_getent (probe_in, "path",      1);
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
-        
+
         if (path == NULL || filename == NULL) {
                 *err = PROBE_ENOELM;
-                
+
                 SEXP_free (behaviors);
                 SEXP_free (path);
                 SEXP_free (filename);
-                
+
                 return (NULL);
         }
-        
+
         if (behaviors == NULL) {
                 SEXP_t *bh_list;
-                
+
                 bh_list = probe_ent_creat ("behaviors",
                                            r0 = probe_attr_creat ("max_depth",
                                                                   r1 = SEXP_string_newf ("-1"),
@@ -359,9 +358,9 @@ SEXP_t *probe_main (SEXP_t *probe_in, int *err, void *mutex)
                                                                   NULL),
                                            NULL /* val */,
                                            NULL /* end */);
-                
+
                 behaviors = SEXP_list_first (bh_list);
-         
+
                 SEXP_vfree (bh_list,
                             r0, r1, r2, r3, r4, NULL);
         } else {
@@ -370,28 +369,28 @@ SEXP_t *probe_main (SEXP_t *probe_in, int *err, void *mutex)
                                             "max_depth", r0 = SEXP_string_newf ("-1"));
                         SEXP_free (r0);
                 }
-                
+
                 if (!probe_ent_attrexists (behaviors, "recurse")) {
                         probe_ent_attr_add (behaviors,
                                             "recurse", r0 = SEXP_string_newf ("symlinks and directories"));
                         SEXP_free (r0);
                 }
-                
+
                 if (!probe_ent_attrexists (behaviors, "recurse_direction")) {
                         probe_ent_attr_add (behaviors,
                                             "recurse_direction", r0 = SEXP_string_newf ("none"));
                         SEXP_free (r0);
                 }
-                
+
                 if (!probe_ent_attrexists (behaviors, "recurse_file_system")) {
                         probe_ent_attr_add (behaviors,
                                             "recurse_file_system", r0 = SEXP_string_newf ("all"));
                         SEXP_free (r0);
                 }
         }
-        
+
         _A(behaviors != NULL);
-        
+
         items = SEXP_list_new (NULL);
 
         switch (pthread_mutex_lock (&__file_probe_mutex)) {
@@ -399,27 +398,27 @@ SEXP_t *probe_main (SEXP_t *probe_in, int *err, void *mutex)
                 break;
         default:
                 _D("Can't lock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
-                
+
                 *err = PROBE_EFATAL;
                 return (NULL);
         }
 
         cbargs.items = items;
         cbargs.filename_ent = filename;
-        
+
         filecnt = find_files (path, filename, behaviors, &file_cb, &cbargs);
         *err    = 0;
-        
+
         if (filecnt < 0) {
                 /* error */
                 SEXP_free (items);
-                r0    = probe_obj_creat ("file_item", NULL,
-                                        /* entities */                                        
+                r0    = probe_item_creat ("file_item", NULL,
+                                        /* entities */
                                         "path",     NULL, path,
                                          NULL);
-                
+
                 probe_item_setstatus(r0, OVAL_STATUS_ERROR);
-                
+
                 SEXP_list_add (items, r0);
                 SEXP_free (r0);
 
@@ -430,30 +429,29 @@ SEXP_t *probe_main (SEXP_t *probe_in, int *err, void *mutex)
                         r0   = probe_obj_creat ("file_item", NULL,
                                                 "path",     NULL, path,
                                                 NULL);
-                        
+
                         probe_item_setstatus(r0, OVAL_STATUS_DOESNOTEXIST);
                         probe_itement_setstatus(r0, "path", 1, OVAL_STATUS_DOESNOTEXIST);
-                        
+
                         SEXP_list_add (items, r0);
                         SEXP_free (r0);
                 }
 		*/
         }
-        
+
         SEXP_free (behaviors);
         SEXP_free (path);
         SEXP_free (filename);
-        
+
         switch (pthread_mutex_unlock (&__file_probe_mutex)) {
         case 0:
                 break;
         default:
                 _D("Can't unlock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
-                
+
                 *err = PROBE_EFATAL;
                 return (NULL);
         }
-        
+
         return (items);
 }
-#endif
