@@ -1,8 +1,8 @@
 /**
  * @addtogroup OVAL
  * @{
- * @addtogroup OVALSYS OVAL System Characteristics
- * Public interface for OVAL System Characteristics.
+ * @addtogroup OVALSYS
+ * Interface for System Characteristics model.
  * @{
  *
  * Class diagram
@@ -14,7 +14,7 @@
  */
 
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -189,10 +189,11 @@ struct oval_variable_binding_iterator;
 
 
 /**
- * Load OVAL system characteristics from a file.
+ * Import the content from a specified XML stream into a oval_syschar_model, return -1 if an error occurred.
+ * If the input_source specifies a model entity that is already registered within the model its content is overwritten.
  * @memberof oval_syschar_model
  */
-int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_import_source *source, void *user_arg);
+int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_import_source *source);
 /**
  * Create new oval_syschar_model.
  * The new model is bound to a specified oval_definition_model and variable bindings.
@@ -310,12 +311,25 @@ bool oval_syschar_model_is_locked(struct oval_syschar_model *syschar_model);
  * @{
  */
 /**
- * Probe oval_objects bound to oval_syschar_model
+ * Probe all oval_objects from oval_definition model that was bound to specific oval_syschar_model. Sucessfull 
+ * finish of this functions result in oval_syschar_model filled out by system characteristic to each object.
  * @memberof oval_syschar_model
+ * return 0 on sucess and -1 on fail. Use \ref ERRORS mechanism to examine the error.
  */
-void oval_syschar_model_probe_objects(struct oval_syschar_model *);
+int oval_syschar_model_probe_objects(struct oval_syschar_model *);
+/**
+ * Gather system information and add it to oval_syschar_model.
+ * @memberof oval_syschar_model
+ * @return 0 on sucess and -1 on fail. Use \ref ERRORS mechanism to examine the error.
+ */
+int oval_syschar_model_probe_sysinfo(struct oval_syschar_model *);
 /**
  * Return <b>true</b> if the syschar_model instance is valid
+ * An oval_syschar_model is valid if all the following are true
+ * 	- The sysinfo attribute is valid
+ * 	- The associated oval_definition_model is valid
+ * 	- All included syschars are valid
+ * 	- All included variable_bindings are valid
  * @memberof oval_syschar_model
  */
 bool oval_syschar_model_is_valid(struct oval_syschar_model *syschar_model);
@@ -546,6 +560,10 @@ void oval_syschar_iterator_free(struct oval_syschar_iterator *);
  */
 /**
  * Return <b>true</b> if the syschar instance is valid
+ * An oval_syschar is valid if all the following are true
+ * 	- The object attribute is valid
+ * 	- All included variable_bindings are valid
+ * 	- All included sysdata are valid
  * @memberof oval_syschar
  */
 bool oval_syschar_is_valid(struct oval_syschar *syschar);
@@ -757,6 +775,8 @@ void oval_sysdata_iterator_free(struct oval_sysdata_iterator *);
  */
 /**
  * Return <b>true</b> if the sysdata instance is valid
+ * An oval_sysdata is valid if all the following are true
+ * 	- All included sysitems are valid
  * @memberof oval_sysdata
  */
 bool oval_sysdata_is_valid(struct oval_sysdata *sysdata);
