@@ -1,5 +1,6 @@
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright (C) 2010 Tresys Technology, LLC
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -18,11 +19,13 @@
  *
  * Authors:
  *      Lukas Kuklinek <lkuklinek@redhat.com>
+ *      Josh Adams <jadams@tresys.com>
  */
 
 #include "item.h"
 #include "elements.h"
 #include "helpers.h"
+#include "xccdf_impl.h"
 #include <assert.h>
 #include <string.h>
 
@@ -35,7 +38,7 @@ bool xccdf_content_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 
 	switch (parent->type) {
 	case XCCDF_BENCHMARK:
-		list = parent->sub.bench.content;
+		list = parent->sub.benchmark.content;
 		break;
 	case XCCDF_GROUP:
 		list = parent->sub.group.content;
@@ -65,19 +68,6 @@ bool xccdf_content_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 	return false;
 }
 
-static char *xccdf_strsep(char **str, char delim)
-{
-	if (str == NULL || *str == NULL)
-		return NULL;
-	char *ret = *str;
-	*str = strchr(*str, delim);
-	if (*str) {
-		**str = '\0';
-		(*str)++;
-	}
-	return ret;
-}
-
 static void xccdf_deps_get(struct xccdf_item *item, struct oscap_list **conflicts, struct oscap_list **requires)
 {
 	switch (item->type) {
@@ -109,7 +99,7 @@ static bool xccdf_item_parse_deps(xmlTextReaderPtr reader, struct xccdf_item *it
 			struct oscap_list *reqs = oscap_list_new();
 			char *ids = xccdf_attribute_copy(reader, XCCDFA_IDREF), *idsstr = ids, *id;
 
-			while ((id = xccdf_strsep(&ids, ' ')) != NULL) {
+			while ((id = oscap_strsep(&ids, " ")) != NULL) {
 				if (strcmp(id, "") == 0) continue;
 				oscap_list_add(reqs, oscap_strdup(id));
 			}
@@ -176,7 +166,7 @@ static void xccdf_item_dump_deps(struct xccdf_item *item, int depth)
 	}
 }
 
-struct xccdf_item *xccdf_group_new(struct xccdf_item *parent)
+struct xccdf_item *xccdf_group_new_internal(struct xccdf_item *parent)
 {
 	struct xccdf_item *group = xccdf_item_new(XCCDF_GROUP, parent);
 	group->sub.group.content = oscap_list_new();
@@ -186,11 +176,26 @@ struct xccdf_item *xccdf_group_new(struct xccdf_item *parent)
 	return group;
 }
 
+struct xccdf_group *xccdf_group_new(void)
+{
+    return XGROUP(xccdf_group_new_internal(NULL));
+}
+
+struct xccdf_group * xccdf_group_clone(const struct xccdf_group * group)
+{
+	struct xccdf_item *new_group = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_group_item));
+	struct xccdf_item *old = XITEM(group);
+	new_group->item = *(xccdf_item_base_clone(&(old->item)));
+	new_group->type = old->type;
+	new_group->sub.group = *(xccdf_group_item_clone(&(old->sub.group), new_group));
+	return XGROUP(new_group);
+}
+
 struct xccdf_item *xccdf_group_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 {
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_GROUP);
 
-	struct xccdf_item *group = xccdf_group_new(parent);
+	struct xccdf_item *group = xccdf_group_new_internal(parent);
 
 	if (!xccdf_item_process_attributes(group, reader)) {
 		xccdf_group_free(group);
@@ -252,7 +257,7 @@ void xccdf_group_free(struct xccdf_item *group)
 	}
 }
 
-struct xccdf_item *xccdf_rule_new(struct xccdf_item *parent)
+struct xccdf_item *xccdf_rule_new_internal(struct xccdf_item *parent)
 {
 	struct xccdf_item *rule = xccdf_item_new(XCCDF_RULE, parent);
 	rule->sub.rule.role = XCCDF_ROLE_FULL;
@@ -267,11 +272,26 @@ struct xccdf_item *xccdf_rule_new(struct xccdf_item *parent)
 	return rule;
 }
 
+struct xccdf_rule *xccdf_rule_new(void)
+{
+    return XRULE(xccdf_rule_new_internal(NULL));
+}
+
+struct xccdf_rule *xccdf_rule_clone(const struct xccdf_rule * rule)
+{
+	struct xccdf_item *new_rule = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_rule_item));
+	struct xccdf_item *old = XITEM(rule);
+	new_rule->item = *(xccdf_item_base_clone(&(old->item)));
+	new_rule->type = old->type;
+	new_rule->sub.rule = *(xccdf_rule_item_clone(&(old->sub.rule)));
+	return XRULE(new_rule);
+}
+
 struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 {
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_RULE);
 
-	struct xccdf_item *rule = xccdf_rule_new(parent);
+	struct xccdf_item *rule = xccdf_rule_new_internal(parent);
 
 	if (!xccdf_item_process_attributes(rule, reader)) {
 		xccdf_rule_free(rule);
@@ -295,14 +315,14 @@ struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *
 				const char *tag = xccdf_attribute_get(reader, XCCDFA_TAG);
 				if (tag == NULL)
 					break;
-				struct xccdf_profile_note *note = oscap_calloc(1, sizeof(struct xccdf_profile_note));
+				struct xccdf_profile_note *note = xccdf_profile_note_new();
 				note->reftag = strdup(tag);
 				note->text = oscap_text_new_parse(XCCDF_TEXT_PROFNOTE, reader);
 				oscap_list_add(rule->sub.rule.profile_notes, note);
 				break;
 			}
 		case XCCDFE_CHECK:{
-				struct xccdf_check *check = xccdf_check_parse(reader, rule);
+				struct xccdf_check *check = xccdf_check_parse(reader);
 				if (check == NULL)
 					break;
 				if (check->selector == NULL || strcmp(check->selector, "") == 0)
@@ -311,10 +331,10 @@ struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *
 				break;
 			}
 		case XCCDFE_FIX:
-			oscap_list_add(rule->sub.rule.fixes, xccdf_fix_parse(reader, rule));
+			oscap_list_add(rule->sub.rule.fixes, xccdf_fix_parse(reader));
 			break;
 		case XCCDFE_FIXTEXT:
-			oscap_list_add(rule->sub.rule.fixtexts, xccdf_fixtext_parse(reader, rule));
+			oscap_list_add(rule->sub.rule.fixtexts, xccdf_fixtext_parse(reader));
 			break;
 		case XCCDFE_IDENT:
 			oscap_list_add(rule->sub.rule.idents, xccdf_ident_parse(reader));
@@ -358,11 +378,22 @@ void xccdf_rule_free(struct xccdf_item *rule)
 	}
 }
 
-struct xccdf_ident *xccdf_ident_new(const char *id, const char *sys)
+struct xccdf_ident *xccdf_ident_new(void)
 {
-	if (id == NULL || sys == NULL)
-		return NULL;
-	struct xccdf_ident *ident = oscap_calloc(1, sizeof(struct xccdf_ident));
+    return oscap_calloc(1, sizeof(struct xccdf_ident));
+}
+
+struct xccdf_ident * xccdf_ident_clone(const struct xccdf_ident * ident)
+{
+	struct xccdf_ident * clone = xccdf_ident_new();
+	clone->id = oscap_strdup(ident->id);
+	clone->system = oscap_strdup(ident->system);
+	return clone;
+}
+
+struct xccdf_ident *xccdf_ident_new_fill(const char *id, const char *sys)
+{
+	struct xccdf_ident *ident = xccdf_ident_new();
 	ident->id = strdup(id);
 	ident->system = strdup(sys);
 	return ident;
@@ -373,7 +404,7 @@ struct xccdf_ident *xccdf_ident_parse(xmlTextReaderPtr reader)
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_IDENT);
 	const char *sys = xccdf_attribute_get(reader, XCCDFA_SYSTEM);
 	const char *id = oscap_element_string_get(reader);
-	return xccdf_ident_new(id, sys);
+	return xccdf_ident_new_fill(id, sys);
 }
 
 void xccdf_ident_dump(struct xccdf_ident *ident, int depth)
@@ -391,6 +422,19 @@ void xccdf_ident_free(struct xccdf_ident *ident)
 	}
 }
 
+struct xccdf_profile_note *xccdf_profile_note_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_profile_note));
+}
+
+struct xccdf_profile_note * xccdf_profile_note_clone(const struct xccdf_profile_note * note)
+{
+	struct xccdf_profile_note * clone = xccdf_profile_note_new();
+	clone->reftag = oscap_strdup(note->reftag);
+	clone->text = oscap_text_clone(note->text);
+	return clone;
+}
+
 void xccdf_profile_note_free(struct xccdf_profile_note *note)
 {
 	if (note) {
@@ -402,10 +446,9 @@ void xccdf_profile_note_free(struct xccdf_profile_note *note)
 
 XCCDF_GENERIC_GETTER(const char *, ident, id) XCCDF_GENERIC_GETTER(const char *, ident, system)
 
-struct xccdf_check *xccdf_check_new(struct xccdf_item *parent)
+struct xccdf_check *xccdf_check_new(void)
 {
 	struct xccdf_check *check = oscap_calloc(1, sizeof(struct xccdf_check));
-	check->parent = parent;
 	check->content_refs = oscap_list_new();
 	check->imports = oscap_list_new();
 	check->exports = oscap_list_new();
@@ -413,18 +456,65 @@ struct xccdf_check *xccdf_check_new(struct xccdf_item *parent)
 	return check;
 }
 
+/* Performs a deep copy of a provided xccdf_check, returns a pointer to that copy */
+struct xccdf_check *xccdf_check_clone(const struct xccdf_check* old_check)
+{
+	struct xccdf_check *new_check = oscap_calloc(1, sizeof(struct xccdf_check));
+
+	new_check->id = oscap_strdup(old_check->id);
+	new_check->system = oscap_strdup(old_check->system);
+	new_check->selector = oscap_strdup(old_check->selector);
+	new_check->content =  oscap_strdup(old_check->content);
+	new_check->oper = old_check->oper;
+
+	new_check->imports = oscap_list_clone(old_check->imports, (oscap_clone_func) xccdf_check_import_clone);
+	new_check->exports = oscap_list_clone(old_check->exports, (oscap_clone_func) xccdf_check_export_clone);
+	new_check->content_refs = oscap_list_clone(old_check->content_refs, (oscap_clone_func) xccdf_check_content_ref_clone);
+	new_check->children = oscap_list_clone(old_check->children, (oscap_clone_func) xccdf_check_clone);
+
+	return new_check;
+}
+
+/* Performs a deep copy of a provided xccdf_check_import, returns a pointer to that copy */
+struct xccdf_check_import *xccdf_check_import_clone(const struct xccdf_check_import* old_import)
+{
+	struct xccdf_check_import *new_import = xccdf_check_import_new();
+	new_import->name = oscap_strdup(old_import->name);
+	new_import->content = oscap_strdup(old_import->content);
+	return new_import;
+}
+
+/* Performs a deep copy of a provided xccdf_check_export, returns a pointer to that copy */
+struct xccdf_check_export *xccdf_check_export_clone(const struct xccdf_check_export* old_export)
+{
+	struct xccdf_check_export *new_export = xccdf_check_export_new();
+	new_export->name = oscap_strdup(old_export->name);
+	new_export->value = oscap_strdup(old_export->value);
+	return new_export;
+}
+
+/* Performs a deep copy of a provided xcdf_check_content_ref, returns a pointer to that copy */
+struct xccdf_check_content_ref *xccdf_check_content_ref_clone(const struct xccdf_check_content_ref* old_ref)
+{
+	struct xccdf_check_content_ref *new_ref = xccdf_check_content_ref_new();
+	new_ref->name = oscap_strdup(old_ref->name);
+	new_ref->href = oscap_strdup(old_ref->href);
+	return new_ref;
+}
+
+
 static const struct oscap_string_map XCCDF_BOOLOP_MAP[] = {
 	{XCCDF_OPERATOR_AND, "and"}, {XCCDF_OPERATOR_AND, "AND"},
 	{XCCDF_OPERATOR_OR, "or"}, {XCCDF_OPERATOR_OR, "OR"},
 	{0, NULL}
 };
 
-struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
+struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader)
 {
 	xccdf_element_t el = xccdf_element_get(reader);
 	if (el != XCCDFE_CHECK && el != XCCDFE_COMPLEX_CHECK)
 		return NULL;
-	struct xccdf_check *check = xccdf_check_new(parent);
+	struct xccdf_check *check = xccdf_check_new();
 
 	check->id = xccdf_attribute_copy(reader, XCCDFA_ID);
 	check->system = xccdf_attribute_copy(reader, XCCDFA_SYSTEM);
@@ -441,14 +531,13 @@ struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader, struct xccdf_item
 		case XCCDFE_COMPLEX_CHECK:
 			if (check->oper == 0)
 				break;
-			oscap_list_add(check->children, xccdf_check_parse(reader, parent));
+			oscap_list_add(check->children, xccdf_check_parse(reader));
 			break;
 		case XCCDFE_CHECK_CONTENT_REF:{
 				const char *href = xccdf_attribute_get(reader, XCCDFA_HREF);
 				if (href == NULL)
 					break;
-				struct xccdf_check_content_ref *ref =
-				    oscap_calloc(1, sizeof(struct xccdf_check_content_ref));
+				struct xccdf_check_content_ref *ref = xccdf_check_content_ref_new();
 				ref->name = xccdf_attribute_copy(reader, XCCDFA_NAME);
 				ref->href = strdup(href);
 				oscap_list_add(check->content_refs, ref);
@@ -462,7 +551,7 @@ struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader, struct xccdf_item
 				const char *name = xccdf_attribute_get(reader, XCCDFA_IMPORT_NAME);
 				if (name == NULL)
 					break;
-				struct xccdf_check_import *imp = oscap_calloc(1, sizeof(struct xccdf_check_import));
+				struct xccdf_check_import *imp = xccdf_check_import_new();
 				imp->name = strdup(name);
 				imp->content = oscap_element_string_copy(reader);
 				oscap_list_add(check->imports, imp);
@@ -472,7 +561,7 @@ struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader, struct xccdf_item
 				const char *name = xccdf_attribute_get(reader, XCCDFA_EXPORT_NAME);
 				if (name == NULL)
 					break;
-				struct xccdf_check_export *exp = oscap_calloc(1, sizeof(struct xccdf_check_export));
+				struct xccdf_check_export *exp = xccdf_check_export_new();
 				exp->name = strdup(name);
 				exp->value = xccdf_attribute_copy(reader, XCCDFA_VALUE_ID);
 				oscap_list_add(check->exports, exp);
@@ -536,6 +625,11 @@ void xccdf_check_content_ref_dump(struct xccdf_check_content_ref *ref, int depth
 	printf("%s (%s)\n", ref->href, ref->name);
 }
 
+struct xccdf_check_content_ref *xccdf_check_content_ref_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_check_content_ref));
+}
+
 void xccdf_check_content_ref_free(struct xccdf_check_content_ref *ref)
 {
 	if (ref) {
@@ -545,6 +639,11 @@ void xccdf_check_content_ref_free(struct xccdf_check_content_ref *ref)
 	}
 }
 
+struct xccdf_check_import *xccdf_check_import_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_check_import));
+}
+
 void xccdf_check_import_free(struct xccdf_check_import *item)
 {
 	if (item) {
@@ -554,6 +653,11 @@ void xccdf_check_import_free(struct xccdf_check_import *item)
 	}
 }
 
+struct xccdf_check_export *xccdf_check_export_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_check_export));
+}
+
 void xccdf_check_export_free(struct xccdf_check_export *item)
 {
 	if (item) {
@@ -583,9 +687,34 @@ const struct oscap_string_map XCCDF_STRATEGY_MAP[] = {
 	fix->content    = oscap_element_string_copy(reader); \
 	} while (false)
 
-struct xccdf_fix *xccdf_fix_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
+struct xccdf_fix *xccdf_fix_new(void)
 {
-	struct xccdf_fix *fix = oscap_calloc(1, sizeof(struct xccdf_fix));
+        return oscap_calloc(1, sizeof(struct xccdf_fix));
+}
+
+/* Creates a deep copy of a provided xccdf_fix, returns a pointer to that copy */
+struct xccdf_fix *xccdf_fix_clone(const struct xccdf_fix *old_fix)
+{
+	struct xccdf_fix *new_fix = oscap_calloc(1, sizeof(struct xccdf_fix));
+
+	new_fix->reboot = old_fix->reboot;
+	new_fix->strategy = old_fix->strategy;
+	new_fix->disruption = old_fix->disruption;
+	new_fix->complexity = old_fix->complexity;
+
+	new_fix->id = oscap_strdup(old_fix->id);
+	new_fix->content = oscap_strdup(old_fix->content);
+	new_fix->system = oscap_strdup(old_fix->system);
+	new_fix->platform = oscap_strdup(old_fix->platform);
+
+	return new_fix;
+}
+
+
+
+struct xccdf_fix *xccdf_fix_parse(xmlTextReaderPtr reader)
+{
+	struct xccdf_fix *fix = xccdf_fix_new();
 	fix->id = xccdf_attribute_copy(reader, XCCDFA_ID);
 	fix->system = xccdf_attribute_copy(reader, XCCDFA_SYSTEM);
 	fix->platform = xccdf_attribute_copy(reader, XCCDFA_PLATFORM);
@@ -593,9 +722,26 @@ struct xccdf_fix *xccdf_fix_parse(xmlTextReaderPtr reader, struct xccdf_item *pa
 	return fix;
 }
 
-struct xccdf_fixtext *xccdf_fixtext_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
+struct xccdf_fixtext *xccdf_fixtext_new(void)
 {
-	struct xccdf_fixtext *fix = oscap_calloc(1, sizeof(struct xccdf_fixtext));
+    return oscap_calloc(1, sizeof(struct xccdf_fixtext));
+}
+
+struct xccdf_fixtext * xccdf_fixtext_clone(const struct xccdf_fixtext * fixtext)
+{
+	struct xccdf_fixtext * clone = xccdf_fixtext_new();
+	clone->reboot = fixtext->reboot;
+	clone->strategy = fixtext->strategy;
+	clone->disruption = fixtext->disruption;
+	clone->complexity = fixtext->complexity;
+	clone->fixref = oscap_strdup(fixtext->fixref);
+	clone->content = oscap_strdup(fixtext->content);
+	return clone;	
+}
+
+struct xccdf_fixtext *xccdf_fixtext_parse(xmlTextReaderPtr reader)
+{
+	struct xccdf_fixtext *fix = xccdf_fixtext_new();
 	fix->fixref = xccdf_attribute_copy(reader, XCCDFA_FIXREF);
 	XCCDF_FIXCOMMON_PARSE(reader, fix);
 	return fix;
@@ -653,52 +799,231 @@ struct oscap_stringlist_iterator *xccdf_group_get_requires(const struct xccdf_gr
 	return oscap_iterator_new(XITEM(group)->sub.group.requires);
 }
 
+void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc, xmlNode *parent)
+{
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+
+	/* Handle Attributes */
+	const char *extends = xccdf_rule_get_extends(rule);
+	if (extends)
+		xmlNewProp(rule_node, BAD_CAST "extends", BAD_CAST extends);
+
+	if (xccdf_rule_get_multiple(rule))
+		xmlNewProp(rule_node, BAD_CAST "multiple", BAD_CAST "True");
+
+	if (xccdf_rule_get_selected(rule))
+		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "True");
+	else
+		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "False");
+
+	float weight = xccdf_rule_get_weight(rule);
+	char weight_str[10];
+	sprintf(weight_str, "%f", weight);
+	xmlNewProp(rule_node, BAD_CAST "weight", BAD_CAST weight_str);
+
+	xccdf_role_t role = xccdf_rule_get_role(rule);
+	xmlNewProp(rule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
+
+	xccdf_level_t severity = xccdf_rule_get_severity(rule);
+	xmlNewProp(rule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
+
+	/* Handle Child Nodes */
+	struct oscap_text_iterator *rationales = xccdf_rule_get_rationale(rule);
+	while (oscap_text_iterator_has_more(rationales)) {
+		struct oscap_text *rationale = oscap_text_iterator_next(rationales);
+		xmlNode * child = xmlNewChild(rule_node, ns_xccdf, BAD_CAST "rationale", BAD_CAST oscap_text_get_text(rationale));
+                if (oscap_text_get_lang(rationale) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(rationale));
+	}
+	oscap_text_iterator_free(rationales);
+
+	struct oscap_string_iterator *platforms = xccdf_rule_get_platforms(rule);
+	while (oscap_string_iterator_has_more(platforms)) {
+		const char *platform = oscap_string_iterator_next(platforms);
+		xmlNewChild(rule_node, ns_xccdf, BAD_CAST "platform", BAD_CAST platform);
+	}
+	oscap_string_iterator_free(platforms);
+
+	struct oscap_stringlist_iterator *lists = xccdf_rule_get_requires(rule);
+	while (oscap_stringlist_iterator_has_more(lists)) {
+		struct oscap_stringlist *list = oscap_stringlist_iterator_next(lists);
+		struct oscap_string_iterator *strings = oscap_stringlist_get_strings(list);
+		while (oscap_string_iterator_has_more(strings)) {
+			const char *requires = oscap_string_iterator_next(strings);
+			xmlNewChild(rule_node, ns_xccdf, BAD_CAST "requires", BAD_CAST requires);
+		}
+		oscap_string_iterator_free(strings);
+	}
+	oscap_stringlist_iterator_free(lists);
+
+	struct oscap_string_iterator *conflicts = xccdf_rule_get_conflicts(rule);
+	while (oscap_string_iterator_has_more(conflicts)) {
+		const char *conflict = oscap_string_iterator_next(conflicts);
+		xmlNewChild(rule_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST conflict);
+	}
+	oscap_string_iterator_free(conflicts);
+
+	struct xccdf_ident_iterator *idents = xccdf_rule_get_idents(rule);
+	while (xccdf_ident_iterator_has_more(idents)) {
+		struct xccdf_ident *ident = xccdf_ident_iterator_next(idents);
+		xccdf_ident_to_dom(ident, doc, rule_node);
+	}
+	xccdf_ident_iterator_free(idents);
+
+	struct xccdf_profile_note_iterator *notes = xccdf_rule_get_profile_notes(rule);
+	while (xccdf_profile_note_iterator_has_more(notes)) {
+		struct xccdf_profile_note *note = xccdf_profile_note_iterator_next(notes);
+		xccdf_profile_note_to_dom(note, doc, rule_node);
+	}
+	xccdf_profile_note_iterator_free(notes);
+
+	struct xccdf_fixtext_iterator *fixtexts = xccdf_rule_get_fixtexts(rule);
+	while (xccdf_fixtext_iterator_has_more(fixtexts)) {
+		struct xccdf_fixtext *fixtext = xccdf_fixtext_iterator_next(fixtexts);
+		xccdf_fixtext_to_dom(fixtext, doc, rule_node);
+	}
+	xccdf_fixtext_iterator_free(fixtexts);
+
+	struct xccdf_fix_iterator *fixes = xccdf_rule_get_fixes(rule);
+	while (xccdf_fix_iterator_has_more(fixes)) {
+		struct xccdf_fix *fix = xccdf_fix_iterator_next(fixes);
+		xccdf_fix_to_dom(fix, doc, rule_node);
+	}
+	xccdf_fix_iterator_free(fixes);
+
+	struct xccdf_check_iterator *checks = xccdf_rule_get_checks(rule);
+	while (xccdf_check_iterator_has_more(checks)) {
+		struct xccdf_check *check = xccdf_check_iterator_next(checks);
+		xccdf_check_to_dom(check, doc, rule_node);
+	}
+	xccdf_check_iterator_free(checks);
+}
+
+void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *doc, xmlNode *parent)
+{
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+
+	/* Handle Attributes */
+	const char *extends = xccdf_group_get_extends(group);
+	if (extends)
+		xmlNewProp(group_node, BAD_CAST "extends", BAD_CAST extends);
+
+	if (xccdf_group_get_selected(group))
+		xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "True");
+	else
+		xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "False");
+
+	float weight = xccdf_group_get_weight(group);
+	char weight_str[10];
+	sprintf(weight_str, "%f", weight);
+	xmlNewProp(group_node, BAD_CAST "weight", BAD_CAST weight_str);
+
+	/* Handle Child Nodes */
+	struct oscap_text_iterator *rationales = xccdf_group_get_rationale(group);
+	while (oscap_text_iterator_has_more(rationales)) {
+		struct oscap_text *rationale = oscap_text_iterator_next(rationales);
+		xmlNode * child = xmlNewChild(group_node, ns_xccdf, BAD_CAST "rationale", BAD_CAST oscap_text_get_text(rationale));
+                if (oscap_text_get_lang(rationale) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(rationale));
+	}
+	oscap_text_iterator_free(rationales);
+
+	struct oscap_string_iterator *platforms = xccdf_group_get_platforms(group);
+	while (oscap_string_iterator_has_more(platforms)) {
+		const char *platform = oscap_string_iterator_next(platforms);
+		xmlNewChild(group_node, ns_xccdf, BAD_CAST "platform", BAD_CAST platform);
+	}
+	oscap_string_iterator_free(platforms);
+
+	struct oscap_stringlist_iterator *lists = xccdf_group_get_requires(group);
+	while (oscap_stringlist_iterator_has_more(lists)) {
+		struct oscap_stringlist *list = oscap_stringlist_iterator_next(lists);
+		struct oscap_string_iterator *strings = oscap_stringlist_get_strings(list);
+		while (oscap_string_iterator_has_more(strings)) {
+			const char *requires = oscap_string_iterator_next(strings);
+			xmlNewChild(group_node, ns_xccdf, BAD_CAST "requires", BAD_CAST requires);
+		}
+		oscap_string_iterator_free(strings);
+	}
+	oscap_stringlist_iterator_free(lists);
+
+	struct oscap_string_iterator *conflicts = xccdf_group_get_conflicts(group);
+	while (oscap_string_iterator_has_more(conflicts)) {
+		const char *conflict = oscap_string_iterator_next(conflicts);
+		xmlNewChild(group_node, ns_xccdf, BAD_CAST "conflicts", BAD_CAST conflict);
+	}
+	oscap_string_iterator_free(conflicts);
+
+	struct xccdf_item_iterator *items = xccdf_group_get_content(group);
+	while (xccdf_item_iterator_has_more(items)) {
+		struct xccdf_item *item = xccdf_item_iterator_next(items);
+		if (XGROUP(xccdf_item_get_parent(item)) == group) {
+			xccdf_item_to_dom(item, doc, group_node);
+		}
+	}
+	xccdf_item_iterator_free(items);
+}
 
 XCCDF_STATUS_CURRENT(rule)
 XCCDF_STATUS_CURRENT(group)
 XCCDF_GROUP_IGETTER(item, content)
 
-XCCDF_RULE_GETTER(const char *, impact_metric)
-XCCDF_RULE_GETTER(xccdf_role_t, role)
-XCCDF_RULE_GETTER(xccdf_level_t, severity)
+XCCDF_ITEM_ADDER_REG(group, rule, content)
+XCCDF_ITEM_ADDER_REG(group, group, content)
+XCCDF_ITEM_ADDER_REG(group, value, values)
+
+bool xccdf_group_add_content(struct xccdf_group *rule, struct xccdf_item *item)
+{
+	if (item == NULL) return false;
+	switch (xccdf_item_get_type(item)) {
+		case XCCDF_RULE:  return xccdf_group_add_rule (rule, XRULE(item));
+		case XCCDF_GROUP: return xccdf_group_add_group(rule, XGROUP(item));
+		case XCCDF_VALUE: return xccdf_group_add_value(rule, XVALUE(item));
+		default: return false;
+	}
+}
+
+XCCDF_ACCESSOR_STRING(rule, impact_metric)
+XCCDF_ACCESSOR_SIMPLE(rule, xccdf_role_t, role)
+XCCDF_ACCESSOR_SIMPLE(rule, xccdf_level_t, severity)
 XCCDF_RULE_GETTER(struct xccdf_check *, check)
-XCCDF_RULE_IGETTER(ident, idents)
-XCCDF_RULE_IGETTER(check, checks)
-XCCDF_RULE_IGETTER(profile_note, profile_notes)
-XCCDF_RULE_IGETTER(fix, fixes)
-XCCDF_RULE_IGETTER(fixtext, fixtexts)
+XCCDF_SETTER_GENERIC(rule, check, struct xccdf_check *, xccdf_check_free,)
+XCCDF_LISTMANIP(rule, ident, idents)
+XCCDF_LISTMANIP(rule, check, checks)
+XCCDF_LISTMANIP(rule, profile_note, profile_notes)
+XCCDF_LISTMANIP(rule, fix, fixes)
+XCCDF_LISTMANIP(rule, fixtext, fixtexts)
 XCCDF_ITERATOR_GEN_S(ident)
 
-XCCDF_GENERIC_GETTER(const char *, check, id)
-XCCDF_GENERIC_GETTER(const char *, check, system)
-XCCDF_GENERIC_GETTER(const char *, check, selector)
-XCCDF_GENERIC_GETTER(const char *, check, content)
-XCCDF_GENERIC_GETTER(struct xccdf_rule *, check, parent)
-XCCDF_GENERIC_GETTER(xccdf_bool_operator_t, check, oper)
-XCCDF_GENERIC_IGETTER(check_import, check, imports)
-XCCDF_GENERIC_IGETTER(check_export, check, exports)
-XCCDF_GENERIC_IGETTER(check_content_ref, check, content_refs)
-XCCDF_GENERIC_IGETTER(check, check, children)
+OSCAP_ACCESSOR_STRING(xccdf_check, id)
+OSCAP_ACCESSOR_STRING(xccdf_check, system)
+OSCAP_ACCESSOR_STRING(xccdf_check, selector)
+OSCAP_ACCESSOR_STRING(xccdf_check, content)
+OSCAP_ACCESSOR_SIMPLE(xccdf_bool_operator_t, xccdf_check, oper)
+OSCAP_IGETINS(xccdf_check_import, xccdf_check, imports, import)
+OSCAP_IGETINS(xccdf_check_export, xccdf_check, exports, export)
+OSCAP_IGETINS(xccdf_check_content_ref, xccdf_check, content_refs, content_ref)
+OSCAP_IGETINS(xccdf_check, xccdf_check, children, child)
 XCCDF_ITERATOR_GEN_S(check_content_ref)
 XCCDF_ITERATOR_GEN_S(check_export) XCCDF_ITERATOR_GEN_S(check_import) XCCDF_ITERATOR_GEN_S(check)
 
-XCCDF_GENERIC_GETTER(const char *, profile_note, reftag) XCCDF_GENERIC_GETTER(const char *, profile_note, text)
-XCCDF_GENERIC_GETTER(const char *, check_import, name) XCCDF_GENERIC_GETTER(const char *, check_import, content)
-XCCDF_GENERIC_GETTER(const char *, check_export, name) XCCDF_GENERIC_GETTER(const char *, check_export, value)
-XCCDF_GENERIC_GETTER(const char *, check_content_ref, name) XCCDF_GENERIC_GETTER(const char *, check_content_ref, href)
+OSCAP_ACCESSOR_STRING(xccdf_profile_note, reftag)    OSCAP_ACCESSOR_TEXT(xccdf_profile_note, text)
+OSCAP_ACCESSOR_STRING(xccdf_check_import, name)      OSCAP_ACCESSOR_STRING(xccdf_check_import, content)
+OSCAP_ACCESSOR_STRING(xccdf_check_export, name)      OSCAP_ACCESSOR_STRING(xccdf_check_export, value)
+OSCAP_ACCESSOR_STRING(xccdf_check_content_ref, name) OSCAP_ACCESSOR_STRING(xccdf_check_content_ref, href)
 
-XCCDF_GENERIC_GETTER(xccdf_strategy_t, fixtext, strategy)
-XCCDF_GENERIC_GETTER(xccdf_level_t, fixtext, disruption)
-XCCDF_GENERIC_GETTER(xccdf_level_t, fixtext, complexity)
-XCCDF_GENERIC_GETTER(bool, fixtext, reboot)
-XCCDF_GENERIC_GETTER(const char *, fixtext, content)
-XCCDF_GENERIC_GETTER(struct xccdf_fix *, fixtext, fixref) XCCDF_ITERATOR_GEN_S(fixtext)
+OSCAP_ACCESSOR_SIMPLE(xccdf_strategy_t, xccdf_fixtext, strategy)
+OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, disruption)
+OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, complexity)
+OSCAP_ACCESSOR_SIMPLE(bool, xccdf_fixtext, reboot)
+OSCAP_ACCESSOR_STRING(xccdf_fixtext, content)
+OSCAP_ACCESSOR_STRING(xccdf_fixtext, fixref) XCCDF_ITERATOR_GEN_S(fixtext)
 
-XCCDF_GENERIC_GETTER(xccdf_strategy_t, fix, strategy)
-XCCDF_GENERIC_GETTER(xccdf_level_t, fix, disruption)
-XCCDF_GENERIC_GETTER(xccdf_level_t, fix, complexity)
-XCCDF_GENERIC_GETTER(bool, fix, reboot)
-XCCDF_GENERIC_GETTER(const char *, fix, content)
-XCCDF_GENERIC_GETTER(const char *, fix, system)
-XCCDF_GENERIC_GETTER(const char *, fix, platform) XCCDF_GENERIC_GETTER(const char *, fix, id) XCCDF_ITERATOR_GEN_S(fix)
+OSCAP_ACCESSOR_SIMPLE(xccdf_strategy_t, xccdf_fix, strategy)
+OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fix, disruption)
+OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fix, complexity)
+OSCAP_ACCESSOR_SIMPLE(bool, xccdf_fix, reboot)
+OSCAP_ACCESSOR_STRING(xccdf_fix, content)
+OSCAP_ACCESSOR_STRING(xccdf_fix, system)
+OSCAP_ACCESSOR_STRING(xccdf_fix, platform)
+OSCAP_ACCESSOR_STRING(xccdf_fix, id)
+XCCDF_ITERATOR_GEN_S(fix)
 
