@@ -23,6 +23,7 @@
 #include "item.h"
 #include "elements.h"
 #include "helpers.h"
+#include "xccdf_impl.h"
 #include "../common/elements.h"
 #include <math.h>
 #include <string.h>
@@ -30,7 +31,7 @@
 static oscap_destruct_func xccdf_value_val_get_destructor(xccdf_value_type_t type);
 static struct xccdf_value_val *xccdf_value_val_new(xccdf_value_type_t type);
 
-struct xccdf_item *xccdf_value_new(struct xccdf_item *parent, xccdf_value_type_t type)
+struct xccdf_item *xccdf_value_new_internal(struct xccdf_item *parent, xccdf_value_type_t type)
 {
 	struct xccdf_item *val = xccdf_item_new(XCCDF_VALUE, parent);
 	val->sub.value.type = type;
@@ -41,6 +42,21 @@ struct xccdf_item *xccdf_value_new(struct xccdf_item *parent, xccdf_value_type_t
 	return val;
 }
 
+struct xccdf_value *xccdf_value_new(xccdf_value_type_t type)
+{
+    return XVALUE(xccdf_value_new_internal(NULL, type));
+}
+
+struct xccdf_value * xccdf_value_clone(const struct xccdf_value * value)
+{
+	struct xccdf_item *new_value = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_value_item));
+	struct xccdf_item *old = XITEM(value);
+	new_value->item = *(xccdf_item_base_clone(&(old->item)));
+	new_value->type = old->type;
+	new_value->sub.value = *(xccdf_value_item_clone(&(old->sub.value)));
+	return XVALUE(new_value);
+}
+
 static const struct oscap_string_map XCCDF_VALUE_TYPE_MAP[] = {
 	{XCCDF_TYPE_NUMBER, "number"},
 	{XCCDF_TYPE_STRING, "string"},
@@ -86,7 +102,7 @@ struct xccdf_item *xccdf_value_parse(xmlTextReaderPtr reader, struct xccdf_item
 	if (xccdf_element_get(reader) != XCCDFE_VALUE)
 		return NULL;
 	xccdf_value_type_t type = oscap_string_to_enum(XCCDF_VALUE_TYPE_MAP, xccdf_attribute_get(reader, XCCDFA_TYPE));
-	struct xccdf_item *value = xccdf_value_new(parent, type);
+	struct xccdf_item *value = xccdf_value_new_internal(parent, type);
 
 	value->sub.value.oper = oscap_string_to_enum(XCCDF_OPERATOR_MAP, xccdf_attribute_get(reader, XCCDFA_OPERATOR));
 	value->sub.value.interface_hint =
@@ -151,6 +167,75 @@ struct xccdf_item *xccdf_value_parse(xmlTextReaderPtr reader, struct xccdf_item
 	return value;
 }
 
+void xccdf_value_to_dom(struct xccdf_value *value, xmlNode *value_node, xmlDoc *doc, xmlNode *parent)
+{
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+
+	/* Handle Attributes */
+	const char *extends = xccdf_value_get_extends(value);
+	if (extends)
+		xmlNewProp(value_node, BAD_CAST "extends", BAD_CAST extends);
+
+	xccdf_operator_t operator = xccdf_value_get_oper(value);
+	xmlNewProp(value_node, BAD_CAST "operator", BAD_CAST XCCDF_OPERATOR_MAP[operator - 1].string);
+
+	xccdf_value_type_t type = xccdf_value_get_type(value);
+	xmlNewProp(value_node, BAD_CAST "type", BAD_CAST XCCDF_VALUE_TYPE_MAP[type - 1].string);
+
+	if (xccdf_value_get_interactive(value))
+		xmlNewProp(value_node, BAD_CAST "interactive", BAD_CAST "True");
+
+	xccdf_interface_hint_t hint = xccdf_value_get_interface_hint(value);
+	xmlNewProp(value_node, BAD_CAST "interfaceHint", BAD_CAST XCCDF_IFACE_HINT_MAP[hint - 1].string);
+
+	/* Handle Child Nodes */
+	const char *val_str = xccdf_value_get_value_string(value);
+	xmlNode *val_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "value", BAD_CAST val_str);
+	const char *selector = xccdf_value_get_selector(value);
+	if (selector)
+		xmlNewProp(val_node, BAD_CAST "Selector", BAD_CAST selector);
+
+	const char *defval_str = xccdf_value_get_defval_string(value);
+	xmlNode *defval_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "default", BAD_CAST defval_str);
+	const char *defval_selector = xccdf_value_get_selector(value);
+	if (defval_selector)
+		xmlNewProp(defval_node, BAD_CAST "Selector", BAD_CAST defval_selector);
+
+	const char *match = xccdf_value_get_match(value);
+	xmlNode *match_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "match", BAD_CAST match);
+	const char *match_selector = xccdf_value_get_selector(value);
+	if (match_selector)
+		xmlNewProp(match_node, BAD_CAST "Selector", BAD_CAST match_selector);
+
+	xccdf_numeric lower_val = xccdf_value_get_lower_bound(value);
+	char lower_str[10];
+	*lower_str = '\0';
+	snprintf(lower_str, sizeof(lower_str), "%f", lower_val);
+	xmlNode *lower_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "lower-bound", BAD_CAST lower_str);
+	const char *lower_selector = xccdf_value_get_selector(value);
+	if (lower_selector)
+		xmlNewProp(lower_node, BAD_CAST "Selector", BAD_CAST lower_selector);
+
+	xccdf_numeric upper_val = xccdf_value_get_upper_bound(value);
+	char upper_str[10];
+	*upper_str = '\0';
+	snprintf(upper_str, sizeof(upper_str), "%f", upper_val);
+	xmlNode *upper_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "upper-bound", BAD_CAST upper_str);
+	const char *upper_selector = xccdf_value_get_selector(value);
+	if (upper_selector)
+		xmlNewProp(upper_node, BAD_CAST "Selector", BAD_CAST upper_selector);
+
+	// No support for choices in OpenSCAP
+	
+	struct oscap_string_iterator *sources = xccdf_value_get_sources(value);
+	while (oscap_string_iterator_has_more(sources)) {
+		const char *source = oscap_string_iterator_next(sources);
+		xmlNewChild(value_node, ns_xccdf, BAD_CAST "source", BAD_CAST source);
+	}
+	oscap_string_iterator_free(sources);
+}
+
+
 void xccdf_value_free(struct xccdf_item *val)
 {
 	oscap_htable_free(val->sub.value.values, xccdf_value_val_get_destructor(val->sub.value.type));
@@ -252,6 +337,15 @@ bool xccdf_value_get_set_selector(struct xccdf_item * value, const char *selecto
 	return value->sub.value.value != NULL;
 }
 
+bool xccdf_value_set_oper(struct xccdf_item * value, xccdf_operator_t oper)
+{
+        __attribute__nonnull__(value);
+
+        value->sub.value.oper = oper;
+        return true;
+
+}
+
 const char *xccdf_value_get_defval_string(const struct xccdf_value *value)
 {
 	if (XITEM(value)->sub.value.type != XCCDF_TYPE_STRING || XITEM(value)->sub.value.value == NULL)
@@ -281,6 +375,33 @@ bool xccdf_value_get_defval_boolean(const struct xccdf_value * value)
 	return false;
 }
 
+char *  xccdf_value_get_selected_value(const struct xccdf_value * value)
+{
+        const char * selector = xccdf_value_get_selector(value);
+        char * selected = NULL;
+        if (selector == NULL) /* default value */
+                selector = "";
+        struct xccdf_value_val *val = oscap_htable_get(XITEM(value)->sub.value.values, selector);
+        if (val == NULL)
+            return NULL;
+        switch (XITEM(value)->sub.value.type) {
+            case XCCDF_TYPE_BOOLEAN:
+                    selected = malloc(sizeof(char));
+                    sprintf(selected, "%b", val->value.b);
+                    break;
+            case XCCDF_TYPE_NUMBER:
+                    /* TODO: compatibility issue: what precision should be here ? */
+                    selected = malloc(5*sizeof(char));
+                    sprintf(selected, "%.5f", val->value.n);
+                    break;
+            case XCCDF_TYPE_STRING:
+                    selected = oscap_strdup(val->value.s);
+                    break;
+        }
+        
+        return selected;
+}
+
 const char *xccdf_value_get_value_string(const struct xccdf_value *value)
 {
 	if (XITEM(value)->sub.value.type != XCCDF_TYPE_STRING || XITEM(value)->sub.value.value == NULL)
@@ -403,3 +524,4 @@ XCCDF_STATUS_CURRENT(value)
 XCCDF_VALUE_GETTER(xccdf_value_type_t, type)
 XCCDF_VALUE_GETTER(xccdf_interface_hint_t, interface_hint)
 XCCDF_VALUE_GETTER(xccdf_operator_t, oper) XCCDF_VALUE_GETTER(const char *, selector) XCCDF_SIGETTER(value, sources)
+XCCDF_ITERATOR_GEN_S(value)
