@@ -25,13 +25,15 @@
 #include "_error.h"
 #include "oscap.h"
 #include <string.h>
+#include <ctype.h>
+#include <limits.h>
+#include <stdarg.h>
 
 struct oscap_import_source {
 	oscap_stream_type_t type;
 	char *encoding;
 	char *name;
 };
-OSCAP_GETTER(const char *, oscap_import_source, name) OSCAP_GETTER(oscap_stream_type_t, oscap_import_source, type)
 
 struct oscap_export_target {
 	oscap_stream_type_t type;
@@ -40,6 +42,9 @@ struct oscap_export_target {
 	int indent;
 	char *indent_string;
 };
+
+OSCAP_GETTER(const char *, oscap_import_source, name) OSCAP_GETTER(oscap_stream_type_t, oscap_import_source, type)
+
 OSCAP_GETTER(const char *, oscap_export_target, name)
 OSCAP_GETTER(const char *, oscap_export_target, encoding)
 OSCAP_GETTER(int, oscap_export_target, indent)
@@ -196,7 +201,7 @@ char *oscap_strdup(const char *str)
 	m = strdup(str);
 
 	if ((m == NULL) && (strlen(m) > 0))
-		oscap_seterr_errno(OSCAP_EALLOC, "Memory allocation failed");
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, "Memory allocation failed.");
 
 	return m;
 }
@@ -213,3 +218,95 @@ char *oscap_strsep(char **str, const char *delim)
 	}
 	return ret;
 }
+
+static const size_t CPE_SPLIT_INIT_ALLOC = 8;
+
+char **oscap_split(char *str, const char *delim)
+{
+
+	__attribute__nonnull__(str);
+
+	char **stringp = &str;
+	int alloc = CPE_SPLIT_INIT_ALLOC;
+	char **fields = oscap_alloc(alloc * sizeof(char *));
+	if (!fields)
+		return NULL;
+
+	int i = 0;
+	while (*stringp) {
+		if (i + 2 > alloc) {
+			void *old = fields;
+			alloc *= 2;
+			fields = oscap_realloc(fields, alloc * sizeof(char *));
+			if (fields == NULL) {
+				oscap_free(old);
+				return NULL;
+			}
+		}
+		fields[i++] = oscap_strsep(stringp, delim);
+	}
+	fields[i] = NULL;
+
+	return fields;
+}
+
+
+int oscap_strcmp(const char *s1, const char *s2)
+{
+	if (s1 == NULL) return INT_MIN;
+	if (s2 == NULL) return INT_MAX;
+	return strcmp(s1, s2);
+}
+
+bool oscap_streq(const char *s1, const char *s2)
+{
+	return (oscap_strcmp(s1, s2) == 0);
+}
+
+char *oscap_trim(char *str)
+{
+	int off, i;
+	if (str == NULL)
+		return NULL;
+	for (i = 0; isspace(str[i]); ++i) ;
+	off = i;
+	while (str[i]) {
+		str[i - off] = str[i];
+		++i;
+	}
+	for (i -= off; isspace(str[--i]) && i >= 0;) ;
+	str[++i] = '\0';
+	return str;
+}
+
+char *oscap_vsprintf(const char *fmt, va_list ap)
+{
+    if (fmt == NULL) return NULL;
+
+    char *ret = NULL;
+    va_list args;
+    va_copy(args, ap);
+
+    char foo[2];
+    int length = vsnprintf(foo, 1, fmt, ap);
+    if (length < 0) goto cleanup;
+
+    ret = oscap_alloc(sizeof(char) * (length + 1));
+    vsprintf(ret, fmt, args);
+    assert(ret[length] == '\0');
+
+cleanup:
+    va_end(args);
+    return ret;
+}
+
+char *oscap_sprintf(const char *fmt, ...)
+{
+    char *ret = NULL;
+    va_list ap;
+    va_start(ap, fmt);
+    ret = oscap_vsprintf(fmt, ap);
+    va_end(ap);
+    return ret;
+}
+
