@@ -20,10 +20,6 @@
  *      Lukas Kuklinek <lkuklinek@redhat.com>
  */
 
-/**
- * @file
- * Common openscap utilities.
- */
 
 #ifndef OSCAP_UTIL_H_
 #define OSCAP_UTIL_H_
@@ -50,6 +46,8 @@ struct oscap_export_target;
  */
 #define OSCAP_HIDDEN_END _Pragma("GCC visibility pop")
 
+OSCAP_HIDDEN_START
+
 /**
  * Function pointer to an object destructor.
  */
@@ -338,4 +336,38 @@ char *oscap_strdup(const char *str);
  */
 char *oscap_strsep(char **str, const char *delim);
 
+/**
+ * Split a string.
+ * Split string using given delimiter.
+ * Produces NULL-terminated array of strings.
+ * Modifies its first argument!
+ * @param str String we want to split
+ * @param delim Delimiter of string parts
+ */
+char **oscap_split(char *str, const char *delim);
+
+
+/// Use strcmp on strings, NULL safe
+int oscap_strcmp(const char *s1, const char *s2);
+/// Check for string equality
+bool oscap_streq(const char *s1, const char *s2);
+/// Trim whitespace (modifies its argument!)
+char *oscap_trim(char *str);
+/// Print to a newly allocated string using a va_list.
+char *oscap_vsprintf(const char *fmt, va_list ap);
+/// Print to a newly allocated string using varialbe arguments.
+char *oscap_sprintf(const char *fmt, ...);
+
+#ifndef OSCAP_CONCAT
+# define OSCAP_CONCAT1(a,b) a ## b
+# define OSCAP_CONCAT(a,b) OSCAP_CONCAT1(a,b)
+#endif
+
+#define OSCAP_GSYM(s) OSCAP_CONCAT(___G_, s)
+
+#define protect_errno                                                   \
+        for (int OSCAP_CONCAT(__e,__LINE__)=errno, OSCAP_CONCAT(__s,__LINE__)=1; OSCAP_CONCAT(__s,__LINE__)--; errno=OSCAP_CONCAT(__e,__LINE__))
+
+OSCAP_HIDDEN_END;
+
 #endif				/* OSCAP_UTIL_H_ */
